import os
import sys
import tempfile
import uuid
from typing import Union
from zipfile import ZipFile

import click
import git

from askanna.config import config
from askanna.core.upload import PackageUpload
from askanna.core.utils.file import zip_files_in_dir
from askanna.core.utils.validate import validate_askanna_yml
from askanna.sdk.package import PackageSDK


def package(src: str) -> str:
    pwd_dir_name = os.path.basename(src)
    random_suffix = uuid.uuid4().hex

    # make a temporary directory
    tmpdir = tempfile.mkdtemp(prefix="askanna-package")

    zip_file = os.path.join(
        tmpdir,
        f"{pwd_dir_name}_{random_suffix}.zip",
    )

    if os.path.isfile(os.path.join(src, "askannaignore")):
        ignore_file = os.path.join(src, "askannaignore")
    elif os.path.isfile(os.path.join(src, ".askannaignore")):
        ignore_file = os.path.join(src, ".askannaignore")
    elif os.path.isfile(os.path.join(src, ".gitignore")):
        ignore_file = os.path.join(src, ".gitignore")
    else:
        ignore_file = None

    cwd = os.getcwd()
    os.chdir(src)
    with ZipFile(zip_file, mode="w") as f:
        zip_files_in_dir(".", f, ignore_file)
    os.chdir(cwd)

    return zip_file


def is_project_config_push_ready() -> bool:
    if not config.project.project_config_path:
        click.echo(
            "We cannot upload without a project config path.\n\nThe project path is set by adding an `askanna.yml` "
            "file to your project root directory.\nIf you have an `askanna.yml` file, please check if your working "
            "directory is set to a project (sub)directory with the `askanna.yml` file.",
            err=True,
        )
        return False

    if not config.project.project_suuid:
        click.echo(
            "We cannot upload to AskAnna without the project SUUID set. Did you add a push-target to your "
            "`askanna.yml` file?",
            err=True,
        )
        return False

    return True


def push(overwrite: bool = False, description: Union[str, None] = None) -> bool:
    if not is_project_config_push_ready():
        sys.exit(1)

    if not validate_askanna_yml(config.project.config_dict):
        sys.exit(1)

    if not overwrite:
        # If a package for the project exists, we will not push a new version.
        packages = PackageSDK().list(project_suuid=config.project.project_suuid, number_of_results=1)
        if len(packages) > 0:
            click.echo(
                "We are not pushing your code to AskAnna because this project already has a code package and "
                "overwrite is set to `False`.",
                err=True,
            )
            sys.exit(1)

    project_folder = os.path.dirname(config.project.project_config_path)
    package_archive = package(project_folder)

    # Attach the description to this package upload
    if not description:
        # Try git and use last commit message
        try:
            repo = git.Repo(".")
        except Exception as e:
            click.echo(e, err=True)
        else:
            commit = repo.head.commit
            description = commit.message

    click.echo(f"Uploading '{project_folder}' to AskAnna...")

    uploader = PackageUpload(
        project_suuid=config.project.project_suuid,
        description=description,
    )
    status, _ = uploader.upload(package_archive)
    if status:
        # remove temporary zip-file from the system including the parent temporary folder
        try:
            os.remove(package_archive)
            os.rmdir(os.path.dirname(package_archive))
            click.echo("Successfully pushed the project to AskAnna!")
        except OSError as e:
            click.echo(
                "Pushing your code was successful, but we could not remove the temporary file "
                "used for uploading your code to AskAnna.",
                err=True,
            )
            click.echo(f"The error: {e.strerror}", err=True)
            click.echo(f"You can manually delete the file: {package_archive}", err=True)
    else:
        click.echo("Pushing your code failed.", err=True)
        sys.exit(1)

    return status



import os
import sys
import tempfile

import click

from askanna.cli.run_utils.utils import string_expand_variables
from askanna.config import config
from askanna.core.upload import ArtifactUpload
from askanna.core.utils.file import create_zip_from_paths

HELP = """
Push the artifact of a run to AskAnna
"""

SHORT_HELP = "Push artifact to AskAnna"


@click.command(help=HELP, short_help=SHORT_HELP)
@click.option(
    "--run",
    "run_suuid",
    required=True,
    envvar="AA_RUN_SUUID",
    help="The run SUUID",
)
@click.option(
    "--job-name",
    "job_name",
    required=True,
    envvar="AA_JOB_NAME",
    help="The name of the job",
)
def cli(run_suuid, job_name):
    project_config = config.project.config_dict

    # First check whether we need to create an artifact or not.
    # If output.artifact or output.paths is not specified, we skip this step and report this to the stdout.
    paths_defined = project_config[job_name].get("output", {}).get("artifact", [])

    if not paths_defined:
        click.echo("  Artifact: no `artifact` defined for this job in `askanna.yml`")
        sys.exit(0)
    elif isinstance(paths_defined, str):
        paths_defined = [paths_defined]

    # Expand and translate paths if they are configured with variables
    paths = string_expand_variables(paths_defined)

    click.echo("  Making a zip file with artifact...")

    tempdir = tempfile.mkdtemp(prefix="askanna-artifact")
    zip_file = os.path.join(tempdir, f"artifact_{run_suuid}.zip")
    create_zip_from_paths(filename=zip_file, paths=paths)

    click.echo("  Uploading artifact to AskAnna...")

    try:
        uploader = ArtifactUpload(run_suuid)
        status, msg = uploader.upload(zip_file)
    except Exception as e:
        click.echo(f"  {e}", err=True)
        sys.exit(1)

    if status:
        click.echo(f"  {msg}")
        try:
            os.remove(zip_file)
            os.rmdir(tempdir)
        except OSError as e:
            click.echo("  Pushing the artifact was successful, but the temporary file could not be removed.", err=True)
            click.echo(f"  The error: {e.strerror}", err=True)
            click.echo(f"  You can manually delete the file: {zip_file}", err=True)
        sys.exit(0)
    else:
        click.echo(f"  {msg}", err=True)
        sys.exit(1)



import sys
from pathlib import Path

import click

from askanna import result


@click.group()
def cli1():
    pass


@cli1.command(help="Download a result of a run", short_help="Download a run result")
@click.option("--id", "-i", "run_suuid", prompt="Run SUUID", required=True, type=str, help="Run SUUID")
@click.option("--output", "-o", "output_path", type=click.Path(path_type=Path), help="Filename for the result")
def get(run_suuid, output_path):
    """
    Download a result of a run
    """
    if not output_path:
        filename = result.get_filename(run_suuid)
        output_path = Path(f"result_{run_suuid}_{filename}")

    if output_path.is_dir():
        click.echo("The output argument is a directory. Please provide a filename for the output.", err=True)
        sys.exit(1)

    if output_path.exists():
        click.echo(f"The file '{output_path}' already exists. We will not overwrite the existing file.", err=True)
        sys.exit(1)

    click.echo("Downloading the result has started...")

    try:
        result.download(run_suuid, output_path)
    except Exception as e:
        click.echo(f"Something went wrong while downloading the result:\n  {e}", err=True)
        sys.exit(1)

    click.echo("We have succesfully downloaded the result.")
    click.echo(f"The result is saved in: {output_path}")


cli = click.CommandCollection(
    sources=[cli1],
    help="Download result of a run",
    short_help="Download run result",
)



import datetime
from dataclasses import dataclass
from typing import Dict

from dateutil import parser as dateutil_parser

from .relation import ProjectRelation, WorkspaceRelation


@dataclass
class Variable:
    suuid: str
    name: str
    value: str
    is_masked: bool
    project: ProjectRelation
    workspace: WorkspaceRelation
    created_at: datetime.datetime
    modified_at: datetime.datetime

    @classmethod
    def from_dict(cls, data: Dict) -> "Variable":
        data["created_at"] = dateutil_parser.parse(data["created_at"])
        data["modified_at"] = dateutil_parser.parse(data["modified_at"])

        project = ProjectRelation.from_dict(data["project"])
        del data["project"]
        workspace = WorkspaceRelation.from_dict(data["workspace"])
        del data["workspace"]

        return cls(project=project, workspace=workspace, **data)



from typing import List, Optional

from askanna.core.dataclasses.base import VISIBILITY
from askanna.core.dataclasses.workspace import Workspace
from askanna.core.exceptions import CreateError, DeleteError, GetError, PatchError
from askanna.gateways.api_client import client

from .utils import ListResponse


class WorkspaceListResponse(ListResponse):
    def __init__(self, data: dict):
        super().__init__(data)
        self.results: List[Workspace] = [Workspace.from_dict(workspace) for workspace in data["results"]]

    @property
    def workspaces(self):
        return self.results


class WorkspaceGateway:
    """Management of workspaces in AskAnna
    This is the class which act as the gateway to the API of AskAnna
    """

    def list(
        self,
        page_size: Optional[int] = None,
        cursor: Optional[str] = None,
        order_by: Optional[str] = None,
        search: Optional[str] = None,
        is_member: Optional[bool] = None,
        visibility: Optional[VISIBILITY] = None,
    ) -> WorkspaceListResponse:
        """List all workspaces

        Args:
            page_size (int, optional): Number of workspaces to return per page. Defaults to the default value of
                the backend.
            cursor (str, optional): Cursor to start the page from. Defaults to None.
            order_by (str, optional): Order by field(s). Defaults to the default value of the backend.
            search (str, optional): Search for a specific workspace.
            is_member (bool, optional): Filter on workspaces where the authenticated user is a member.
            visibility ("PRIVATE" or "PUBLIC", optional): Filter on workspaces with a specific visibility.

        Raises:
            GetError: Error based on response status code with the error message from the API

        Returns:
            WorkspaceListResponse: The response from the API with a list of workspaces and pagination information.
        """
        assert page_size is None or page_size > 0, "page_size must be a positive integer"
        assert is_member is None or isinstance(is_member, bool), "is_member must be a boolean"
        if visibility is not None:
            visibility = visibility.lower()  # type: ignore
            assert visibility in ["public", "private"], "visibility must be 'public' or 'private'"

        response = client.get(
            url=client.askanna_url.workspace.workspace_list(),
            params={
                "page_size": page_size,
                "cursor": cursor,
                "order_by": order_by,
                "search": search,
                "is_member": is_member,
                "visibility": visibility,
            },
        )

        if response.status_code != 200:
            error_message = f"{response.status_code} - Something went wrong while retrieving the workspace list"
            try:
                error_message += f":\n  {response.json()}"
            except ValueError:
                pass
            raise GetError(error_message)

        return WorkspaceListResponse(response.json())

    def detail(self, workspace_suuid: str) -> Workspace:
        """Get information of a workspace

        Args:
            workspace_suuid (str): SUUID of the workspace

        Raises:
            GetError: Error based on response status code with the error message from the API

        Returns:
            Workspace: Workspace information in a Workspace dataclass
        """
        response = client.get(
            url=client.askanna_url.workspace.workspace_detail(workspace_suuid=workspace_suuid),
        )

        if response.status_code == 404:
            raise GetError(f"404 - The workspace SUUID '{workspace_suuid}' was not found")
        elif response.status_code != 200:
            raise GetError(
                f"{response.status_code} - Something went wrong while retrieving workspace SUUID '{workspace_suuid}': "
                f"{response.json()}"
            )

        return Workspace.from_dict(response.json())

    def create(self, name: str, description: str = "", visibility: str = "PRIVATE") -> Workspace:
        """Create a new workspace

        Args:
            name (str): Name of the new workspace
            description (str, optional): Description for the new workspace. Defaults to "" (empty string).
            visibility ("PUBLIC" or "PRIVATE", optional): Visibility of the new workspace. Defaults to "PRIVATE".

        Raises:
            ValueError: Error when visibility is not "PUBLIC" or "PRIVATE"
            CreateError: Error based on response status code with the error message from the API

        Returns:
            Workspace: The information of the newly created workspace in a Workspace dataclass
        """
        if visibility and visibility not in ["PUBLIC", "PRIVATE"]:
            raise ValueError("Visibility must be either PUBLIC or PRIVATE")

        response = client.create(
            url=client.askanna_url.workspace.workspace(),
            json={
                "name": name,
                "description": description,
                "visibility": visibility,
            },
        )

        if response.status_code == 201:
            return Workspace.from_dict(response.json())
        else:
            raise CreateError(
                f"{response.status_code} - Something went wrong while creating the workspace: {response.json()}"
            )

    def change(
        self,
        workspace_suuid: str,
        name: Optional[str] = None,
        description: Optional[str] = None,
        visibility: Optional[str] = None,
    ) -> Workspace:
        """Change the name, description and/or visibility of a workspace

        Args:
            workspace_suuid (str): SUUID of the workspace you want to change the information of
            name (str, optional): New name for the workspace. Defaults to None.
            description (str, optional): New description of the workspace. Defaults to None.
            visibility ("PUBLIC" or "PRIVATE", optional): New visibility of the workspace. Defaults to None.

        Raises:
            ValueError: Error when visibility is not "PUBLIC" or "PRIVATE"
            ValueError: Error if none of the arguments 'name', 'description' or 'visibility' are provided
            PatchError: Error based on response status code with the error message from the API

        Returns:
            Workspace: The changed workspace information in a Workspace dataclass
        """
        if visibility and visibility not in ["PUBLIC", "PRIVATE"]:
            raise ValueError("Visibility must be either PUBLIC or PRIVATE")

        changes = {}
        if name:
            changes.update({"name": name})
        if description:
            changes.update({"description": description})
        if visibility:
            changes.update({"visibility": visibility})

        if not changes:
            raise ValueError("At least one of the parameters 'name', 'description' or 'visibility' must be set.")

        response = client.patch(
            url=client.askanna_url.workspace.workspace_detail(workspace_suuid),
            json=changes,
        )

        if response.status_code == 200:
            return Workspace.from_dict(response.json())
        else:
            raise PatchError(
                f"{response.status_code} - Something went wrong while updating the workspace SUUID "
                f"'{workspace_suuid}': {response.json()}"
            )

    def delete(self, workspace_suuid: str) -> bool:
        """Delete a workspace

        Args:
            workspace_suuid (str): SUUID of the workspace you want to delete

        Raises:
            DeleteError: Error based on response status code with the error message from the API

        Returns:
            bool: True if the workspace was succesfully deleted
        """
        response = client.delete(
            url=client.askanna_url.workspace.workspace_detail(workspace_suuid),
        )

        if response.status_code == 204:
            return True
        elif response.status_code == 404:
            raise DeleteError(f"404 - The workspace SUUID '{workspace_suuid}' was not found")
        else:
            raise DeleteError(
                f"{response.status_code} - Something went wrong while deleting the workspace SUUID "
                f"'{workspace_suuid}': {response.json()}"
            )



import datetime
from dataclasses import dataclass
from typing import Dict, Optional

from dateutil import parser as dateutil_parser

from .base import VISIBILITY
from .relation import PackageRelation, UserRelation, WorkspaceRelation


@dataclass
class Project:
    suuid: str
    name: str
    description: str
    visibility: VISIBILITY
    workspace: WorkspaceRelation
    package: Optional[PackageRelation]
    created_by: Optional[UserRelation]
    is_member: bool
    permission: dict
    created_at: datetime.datetime
    modified_at: datetime.datetime

    def __str__(self):
        return f"Project: {self.name} ({self.suuid})"

    @classmethod
    def from_dict(cls, data: Dict) -> "Project":
        data["created_at"] = dateutil_parser.parse(data["created_at"])
        data["modified_at"] = dateutil_parser.parse(data["modified_at"])

        workspace = WorkspaceRelation.from_dict(data["workspace"])
        del data["workspace"]

        package = PackageRelation.from_dict(data["package"]) if data["package"] else None
        del data["package"]

        created_by = UserRelation.from_dict(data["created_by"]) if data["created_by"] else None
        del data["created_by"]

        return cls(workspace=workspace, package=package, created_by=created_by, **data)



import os
import sys

import click
from cookiecutter.exceptions import OutputDirExistsException
from cookiecutter.main import cookiecutter as cookiecreator
from slugify import slugify

from askanna.cli.utils import ask_which_workspace
from askanna.config import config
from askanna.core.dataclasses.project import Project
from askanna.core.push import push
from askanna.sdk.project import ProjectSDK
from askanna.settings import DEFAULT_PROJECT_TEMPLATE

HELP = """
This command will allow you to create an AskAnna project in a new directory
"""

SHORT_HELP = "Create a project in a new directory"


class CreateProject:
    def __init__(self, name: str = ""):
        self.name = name
        if self.name:
            self.slugified_name = slugify(self.name)

    def cli(self, workspace_suuid: str = "", description: str = "") -> Project:
        if not self.name:
            click.echo(
                "Hi! It is time to create a new project in AskAnna. "
                "We start with some information about the project.\n"
            )
            self.name = click.prompt("Project name", type=str)
            self.slugified_name = slugify(self.name)

            if not description:
                description = click.prompt("Project description", type=str, default="", show_default=False)

        if not workspace_suuid:
            workspace = ask_which_workspace("In which workspace do you want to create the new project?")
            workspace_suuid = workspace.suuid

        try:
            project = ProjectSDK().create(workspace_suuid=workspace_suuid, name=self.name, description=description)
        except Exception as e:
            click.echo(f"Something went wrong while creating the project:\n  {e}", err=True)
            sys.exit(1)
        else:
            click.echo(f"\nYou successfully created project '{project.name}' with SUUID '{project.suuid}'")
            return project


@click.command(help=HELP, short_help=SHORT_HELP)
@click.argument("name", required=False)
@click.option(
    "--workspace",
    "-w",
    show_default=True,
    help="Workspace SUUID where you want to create the project",
)
@click.option(
    "--description",
    "-d",
    default="",
    help="Description of the project [optional]",
)
@click.option(
    "--template",
    "-t",
    "project_template",
    help="Location of a Cookiecutter project template",
)
@click.option(
    "--push/--no-push",
    "-p",
    "is_push",
    default=False,
    show_default=False,
    help="Push an initial version of the code [default: no-push]",
)
def cli(name, workspace, description, project_template, is_push):
    cwd = os.getcwd()
    askanna_project_file = os.path.join(cwd, "askanna.yml")
    if os.path.exists(askanna_project_file):
        click.echo(
            "This directory already has an 'askanna.yml' file and we cannot create a new project directory within an "
            "AskAnna project directory.",
            err=True,
        )
        sys.exit(1)

    project_creator = CreateProject(name=name)
    project_info = project_creator.cli(workspace_suuid=workspace, description=description)
    project_dir = project_creator.slugified_name

    if config.server.ui:
        project_url = config.server.ui + "/" + project_info.workspace.suuid + "/project/" + project_info.suuid + "/"
    else:
        project_url = config.server.remote + "/v1/project/" + project_info.suuid + "/"

    try:
        cookiecreator(
            project_template or DEFAULT_PROJECT_TEMPLATE,
            no_input=True,
            overwrite_if_exists=False,
            extra_context={
                "project_name": project_info.name,
                "project_directory": project_dir,
                "project_push_target": project_url,
            },
        )
    except OutputDirExistsException:
        click.echo(
            f"You already have a project directory '{project_dir}'. If you open the new project in AskAnna, you find "
            "instructions about how you can push your project to AskAnna. You can open your project at:"
        )
        click.echo(project_url)
        click.echo("\nSuccess with your project!")
        sys.exit(1)
    else:
        click.echo(f"Open your new local project directory:\n  cd {project_dir}")

        if is_push:
            # Also push the new directory to AskAnna
            click.echo("")
            config.project.reload_config(os.path.join(os.getcwd(), project_dir, "askanna.yml"))
            push(description="Initial push")

    if config.server.ui:
        click.echo("\nYou can find your project in AskAnna at:")
        click.echo(project_url)

    click.echo(
        "\nAs a first step you can configure your first job for this project in the `askanna.yml` file."
        "\nOnes you are done, you can push your code to AskAnna with the command:\n"
        "\n"
        "  askanna push\n"
        "\n"
        "More info about creating jobs: https://docs.askanna.io/job/create-job/\n"
        "\n"
        "Success with your project!"
    )



from pathlib import Path
from typing import List, Optional, Union

from askanna.config import config
from askanna.core.dataclasses.job import Payload
from askanna.core.dataclasses.run import (
    STATUS,
    TRIGGER,
    ArtifactInfo,
    MetricList,
    Run,
    RunStatus,
    VariableList,
)
from askanna.core.exceptions import GetError
from askanna.gateways.run import RunGateway

from .job import JobSDK
from .mixins import ListMixin

__all__ = [
    "RunSDK",
    "ResultSDK",
    "ArtifactSDK",
]


class RunSDK(ListMixin):
    """Management of runs in AskAnna
    This class is a wrapper around the RunGateway and can be used to manage jobs in Python.
    """

    gateway = RunGateway()
    run_suuid = None

    def _get_run_suuid(self) -> str:
        if not self.run_suuid:
            raise ValueError("No run SUUID set")

        return self.run_suuid

    def list(
        self,
        status: Optional[STATUS] = None,
        status__exclude: Optional[STATUS] = None,
        run_suuid_list: Optional[List[str]] = None,
        run_suuid__exclude: Optional[str] = None,
        job_name: Optional[str] = None,
        job_suuid: Optional[str] = None,
        job_suuid__exclude: Optional[str] = None,
        project_suuid: Optional[str] = None,
        project_suuid__exclude: Optional[str] = None,
        workspace_suuid: Optional[str] = None,
        workspace_suuid__exclude: Optional[str] = None,
        created_by_suuid: Optional[str] = None,
        created_by_suuid__exclude: Optional[str] = None,
        trigger: Optional[Union[TRIGGER, List[TRIGGER]]] = None,
        trigger__exclude: Optional[Union[TRIGGER, List[TRIGGER]]] = None,
        package_suuid: Optional[str] = None,
        package_suuid__exclude: Optional[str] = None,
        include_metrics: bool = False,
        include_variables: bool = False,
        number_of_results: int = 100,
        order_by: Optional[str] = None,
        search: Optional[str] = None,
    ) -> List[Run]:
        """List all runs with filter and order options

        Args:
            status (STATUS, optional): Status of the run to filter on. Defaults to None.
            status__exclude (str, optional): Status of the run to exclude. Defaults to None.
              STATUS values: queued, running, finished, failed

            run_suuid_list (List[str], optional): List of run SUUIDs to filter on. Defaults to None.
            run_suuid__exclude (str, optional): SUUID of the run to exclude. Defaults to None.

            job_name (str, optional): Name of the job to filter on. Defaults to None.
            job_suuid (str, optional): SUUID of the job to filter on. Defaults to None.
            job_suuid__exclude (str, optional): SUUID of the job to exclude. Defaults to None.

            project_suuid (str, optional): SUUID of the project to filter on. Defaults to None.
            project_suuid__exclude (str, optional): SUUID of the project to exclude. Defaults to None.

            workspace_suuid (str, optional): SUUID of the workspace to filter on. Defaults to None.
            workspace_suuid__exclude (str, optional): SUUID of the workspace to exclude. Defaults to None.

            created_by_suuid (str, optional): SUUID of the workspace member to filter on. Defaults to None.
            created_by_suuid__exclude (str, optional): SUUID of the workspace member to exclude.
              Defaults to None.

            trigger (TRIGGER, optional): Trigger of the run to filter on. Defaults to None.
            trigger__exclude (TRIGGER, optional): Trigger of the run to exclude. Defaults to None.
              TRIGGER values: api, cli, python-sdk, webui, schedule, worker

            package_suuid (str, optional): SUUID of the package to filter on. Defaults to None.
            package_suuid__exclude (str, optional): SUUID of the package to exclude. Defaults to None.

            include_metrics (bool, optional): Include the metrics in the Run dataclass. Defaults to False.
            include_variables (bool, optional): Include the variables in the Run dataclass. Defaults to False.

            number_of_results (int): Number of runs to return. Defaults to 100.
            order_by (str, optional): Order by field(s).

            search (str, optional): Search for a specific run.

        Raises:
            GetError: Error based on response status code with the error message from the API

        Returns:
            List[Run]: List of runs. List items are of type Run dataclass.
        """

        if job_suuid and job_name:
            raise ValueError("Parameters 'job_suuid' and 'job_name' are both set. Please only set one.")
        if job_name:
            project_suuid = project_suuid or config.project.project_suuid
            job_suuid = JobSDK().get_job_by_name(job_name=job_name, project_suuid=project_suuid).suuid

        run_list = super().list(
            number_of_results=number_of_results,
            order_by=order_by,
            other_query_params={
                "status": status,
                "status__exclude": status__exclude,
                "run_suuid_list": run_suuid_list,
                "run_suuid__exclude": run_suuid__exclude,
                "job_suuid": job_suuid,
                "job_suuid__exclude": job_suuid__exclude,
                "project_suuid": project_suuid,
                "project_suuid__exclude": project_suuid__exclude,
                "workspace_suuid": workspace_suuid,
                "workspace_suuid__exclude": workspace_suuid__exclude,
                "created_by_suuid": created_by_suuid,
                "created_by_suuid__exclude": created_by_suuid__exclude,
                "trigger": trigger,
                "trigger__exclude": trigger__exclude,
                "package_suuid": package_suuid,
                "package_suuid__exclude": package_suuid__exclude,
                "search": search,
            },
        )

        if include_metrics:
            for run in run_list:
                run.metrics = self.gateway.metric(run.suuid)

        if include_variables:
            for run in run_list:
                run.variables = self.gateway.variable(run.suuid)

        return run_list

    def get(
        self,
        run_suuid: Optional[str] = None,
        include_metrics: bool = False,
        include_variables: bool = False,
    ) -> Run:
        """Get information about a run

        Args:
            run_suuid (str, optional): SUUID of the run
            include_metrics (bool, optional): Include the run metrics in the Run dataclass. Defaults to False.
            include_variables (bool, optional): Include the run variables in the Run dataclass. Defaults to False.

        Raises:
            GetError: Error based on response status code with the error message from the API

        Returns:
            Run: Run info in a Run dataclass
        """
        run_suuid = run_suuid or self._get_run_suuid()
        run = self.gateway.detail(run_suuid)

        if include_metrics:
            run.metrics = self.gateway.metric(run.suuid)

        if include_variables:
            run.variables = self.gateway.variable(run.suuid)

        return run

    def change(
        self, run_suuid: Optional[str] = None, name: Optional[str] = None, description: Optional[str] = None
    ) -> Run:
        """Change the name or description of a run

        Args:
            run_suuid (str, optional): SUUID of the run
            name (str, optional): New name of the run. Defaults to None.
            description (str, optional): New description of the run. Defaults to None.

        Raises:
            PatchError: Error based on response status code with the error message from the API

        Returns:
            Run: The updated run in a Run dataclass
        """
        run_suuid = run_suuid or self._get_run_suuid()
        return self.gateway.change(run_suuid=run_suuid, name=name, description=description)

    def delete(self, run_suuid: Optional[str] = None) -> bool:
        """Delete a run

        Args:
            run_suuid (str, optional): SUUID of the run

        Raises:
            DeleteError: Error based on response status code with the error message from the API

        Returns:
            bool: True if the run was deleted
        """
        run_suuid = run_suuid or self._get_run_suuid()
        return self.gateway.delete(run_suuid=run_suuid)

    def start(
        self,
        job_suuid: Optional[str] = None,
        data: Optional[dict] = None,
        name: Optional[str] = None,
        description: Optional[str] = None,
        project_suuid: Optional[str] = None,
        job_name: Optional[str] = None,
    ) -> RunStatus:
        """Start a run

        Args:
            job_suuid (str, optional): SUUID of the job to run. If not set, job_name must be set.
            data (dict, optional): Data to pass to the job. Defaults to None.
            name (str, optional): Name of the run. Defaults to None.
            description (str, optional): Description of the run. Defaults to None.
            project_suuid (str, optional): SUUID of the project to run the job in. Defaults to None.
            job_name (str, optional): Name of the job to run. Defaults to None.

        Raises:
            PostError: Error based on response status code with the error message from the API

        Returns:
            RunStatus: The run status information in a RunStatus dataclass
        """
        run_status = JobSDK().run_request(job_suuid, data, name, description, project_suuid, job_name)
        self.run_suuid = run_status.suuid
        return run_status

    def status(self, run_suuid: Optional[str] = None) -> RunStatus:
        """Get the status of a run

        Args:
            run_suuid (str, optional): SUUID of the run

        Raises:
            GetError: Error based on response status code with the error message from the API

        Returns:
            RunStatus: The run status information in a RunStatus dataclass
        """
        run_suuid = run_suuid or self._get_run_suuid()
        return self.gateway.status(run_suuid)

    def log(self, run_suuid: Optional[str] = None, number_of_lines: int = 100) -> List:
        """Get the log of a run

        Args:
            run_suuid (str, optional): SUUID of the run
            number_of_lines (int, optional): Number of lines to return. Defaults to 100.

        Raises:
            GetError: Error based on response status code with the error message from the API

        Returns:
            List: List of log lines
        """
        run_suuid = run_suuid or self._get_run_suuid()
        return self.gateway.log(run_suuid, limit=number_of_lines)

    def get_metric(self, run_suuid: Optional[str] = None) -> MetricList:
        """Get the metrics of a run

        Args:
            run_suuid (str, optional): SUUID of the run

        Raises:
            GetError: Error based on response status code with the error message from the API

        Returns:
            MetricList: List of metrics
        """
        run_suuid = run_suuid or self._get_run_suuid()
        return self.gateway.metric(run_suuid=run_suuid)

    def get_variable(self, run_suuid: Optional[str] = None) -> VariableList:
        """Get the variables used for a run

        Args:
            run_suuid (str, optional): SUUID of the run

        Raises:
            GetError: Error based on response status code with the error message from the API

        Returns:
            VariableList: List of variables
        """
        run_suuid = run_suuid or self._get_run_suuid()
        return self.gateway.variable(run_suuid=run_suuid)

    def payload(
        self,
        run_suuid: Optional[str] = None,
        payload_suuid: Optional[str] = None,
        output_path: Optional[Union[Path, str]] = None,
    ) -> Union[bytes, None]:
        """Get the payload of a run
        Args:
            run_suuid (str, optional): SUUID of the run
            output_path (Path | str, optional): Path to save the payload to. Defaults to None.

        Raises:
            GetError: Error based on response status code with the error message from the API

        Returns:
            None: if output_path is set, the payload is saved to the output_path and None is returned
            bytes: if output_path is not set, the payload is returned as bytes
        """
        run_suuid = run_suuid or self._get_run_suuid()

        if not payload_suuid:
            payload_info = self.payload_info(run_suuid)

            if not payload_info:
                return

            payload_suuid = payload_info.suuid

        return self.gateway.payload(run_suuid=run_suuid, payload_suuid=payload_suuid, output_path=output_path)

    def payload_info(self, run_suuid: Optional[str] = None) -> Union[Payload, None]:
        """Get the payload info of a run

        Args:
            run_suuid (str, optional): SUUID of the run

        Raises:
            GetError: Error based on response status code with the error message from the API

        Returns:
            Payload: Payload info in a Payload dataclass
            None: If no payload is available for the run
        """
        run_suuid = run_suuid or self._get_run_suuid()
        return self.gateway.payload_info(run_suuid)

    def result(
        self, run_suuid: Optional[str] = None, output_path: Optional[Union[Path, str]] = None
    ) -> Union[bytes, None]:
        """Get the result of a run

        Args:
            run_suuid (str, optional): SUUID of the run
            output_path (Path | str, optional): Path to save the result to. Defaults to None.

        Raises:
            GetError: Error based on response status code with the error message from the API

        Returns:
            None: if output_path is set, the result is saved to the output_path and None is returned
            bytes: if output_path is not set, the result is returned as bytes
        """
        run_suuid = run_suuid or self._get_run_suuid()
        return self.gateway.result(run_suuid, output_path)

    def result_content_type(self, run_suuid: Optional[str] = None) -> str:
        """Get the content type of the result of a run

        Args:
            run_suuid (str, optional): SUUID of the run

        Raises:
            GetError: Error based on response status code with the error message from the API

        Returns:
            str: Content type of the result
        """
        run_suuid = run_suuid or self._get_run_suuid()
        return self.gateway.result_content_type(run_suuid)

    def artifact(
        self, run_suuid: Optional[str] = None, output_path: Optional[Union[Path, str]] = None
    ) -> Union[bytes, None]:
        """Get the artifact of a run

        Args:
            run_suuid (str, optional): SUUID of the run
            output_path (Path | str, optional): Path to save the artifact to. Defaults to None.

        Raises:
            GetError: Error based on response status code with the error message from the API

        Returns:
            None: if output_path is set, the artifact is saved to the output_path and None is returned
            bytes: if output_path is not set, the artifact is returned as bytes
        """
        run_suuid = run_suuid or self._get_run_suuid()
        return self.gateway.artifact(run_suuid=run_suuid, output_path=output_path)

    def artifact_info(self, run_suuid: Optional[str] = None) -> ArtifactInfo:
        """Get the artifact info of a run

        Args:
            run_suuid (str, optional): SUUID of the run

        Raises:
            GetError: Error based on response status code with the error message from the API

        Returns:
            ArtifactInfo: Artifact info in a ArtifactInfo dataclass
        """
        run_suuid = run_suuid or self._get_run_suuid()
        return self.gateway.artifact_info(run_suuid)


class ResultSDK:
    """Get result SDK"""

    def get(self, run_suuid: str) -> Union[bytes, None]:
        """Get the result of a run

        Args:
            run_suuid (str): SUUID of the run

        Returns:
            bytes: The result as bytes
            None: If the run has no result
        """
        return RunSDK().result(run_suuid)

    def download(self, run_suuid: str, output_path: Union[Path, str]) -> None:
        """Download the result of a run

        Args:
            run_suuid (str): SUUID of the run
            output_path (Path | str): Path to save the result to

        Returns:
            None
        """
        RunSDK().result(run_suuid, output_path)

    def get_content_type(self, run_suuid: str) -> str:
        """Get the content type of the result of a run

        Args:
            run_suuid (str): SUUID of the run

        Returns:
            str: Content type of the result
        """
        return RunSDK().result_content_type(run_suuid)

    def get_filename(self, run_suuid: str) -> str:
        """Get the filename of the result of a run

        Args:
            run_suuid (str): SUUID of the run

        Returns:
            str: Filename of the result
        """
        run = RunSDK().get(run_suuid)

        if run.result:
            return run.result.get("name", "")
        else:
            raise GetError(f"No result found for run SUUID '{run_suuid}'")


class ArtifactSDK:
    """Get artifact SDK"""

    def get(self, run_suuid: str) -> Union[bytes, None]:
        """Get the artifact of a run

        Args:
            run_suuid (str): SUUID of the run

        Returns:
            bytes: The artifact as bytes
            None: If the run has no artifact
        """
        return RunSDK().artifact(run_suuid)

    def download(self, run_suuid: str, output_path: Union[Path, str]) -> None:
        """Download the artifact of a run

        Args:
            run_suuid (str): SUUID of the run
            output_path (Path | str): Path to save the artifact to

        Returns:
            None
        """
        RunSDK().artifact(run_suuid, output_path)

    def info(self, run_suuid: str) -> ArtifactInfo:
        """Get the artifact info of a run

        Args:
            run_suuid (str): SUUID of the run

        Returns:
            ArtifactInfo: Artifact info in a ArtifactInfo dataclass
        """
        return RunSDK().artifact_info(run_suuid)



from typing import List, Optional

from askanna.core.dataclasses.variable import Variable
from askanna.core.exceptions import DeleteError, GetError, PatchError, PostError
from askanna.gateways.api_client import client

from .utils import ListResponse


class VariableListResponse(ListResponse):
    def __init__(self, data: dict):
        super().__init__(data)
        self.results: List[Variable] = [Variable.from_dict(variable) for variable in data["results"]]

    @property
    def variables(self):
        return self.results


class VariableGateway:
    """Management of variables in AskAnna
    This is the class which act as gateway to the API of AskAnna
    """

    def list(
        self,
        project_suuid: Optional[str] = None,
        workspace_suuid: Optional[str] = None,
        is_masked: Optional[bool] = None,
        page_size: Optional[int] = None,
        cursor: Optional[str] = None,
        order_by: Optional[str] = None,
        search: Optional[str] = None,
    ) -> VariableListResponse:
        """List variables with the option to filter and order options

        Args:
            project_suuid (str, optional): Project SUUID to filter for variables in a project. Defaults to None.
            workspace_suuid (str, optional): Workspace SUUID to filter for variables in a workspace. Defaults to None.
            is_masked (bool, optional): Filter on variables that are masked or not. Defaults to None.
            page_size (int, optional): Number of results per page. Defaults to the default value of the backend.
            cursor (str, optional): Cursor to start the page from. Defaults to None.
            order_by (str, optional): Order by field(s). Defaults to the default value of the backend.
            search (str, optional): Search for a specific variable.

        Raises:
            GetError: Error based on response status code with the error message from the API

        Returns:
            VariableListResponse: The response from the API with a list of variables and pagination information.
        """
        assert page_size is None or page_size > 0, "page_size must be a positive integer"
        assert is_masked is None or isinstance(is_masked, bool), "is_masked must be a boolean"

        response = client.get(
            url=client.askanna_url.variable.variable(),
            params={
                "project_suuid": project_suuid,
                "workspace_suuid": workspace_suuid,
                "is_masked": is_masked,
                "page_size": page_size,
                "cursor": cursor,
                "order_by": order_by,
                "search": search,
            },
        )

        if response.status_code != 200:
            error_message = response.json().get("detail", "Something went wrong while retrieving variable list")
            try:
                error_message += f": {response.json()}"
            except ValueError:
                pass
            raise GetError(error_message)

        return VariableListResponse(response.json())

    def detail(self, variable_suuid: str) -> Variable:
        """Get the details of a variable

        Args:
            variable_suuid (str): SUUID of the variable

        Returns:
            Variable: A variable dataclass
        """
        response = client.get(url=client.askanna_url.variable.variable_detail(variable_suuid))
        if response.status_code == 404:
            raise GetError(f"{response.status_code} - The variable SUUID '{variable_suuid}' was not found")
        if response.status_code != 200:
            raise GetError(
                f"{response.status_code} - Something went wrong while retrieving the variable SUUID "
                f"'{variable_suuid}': {response.json()}"
            )

        return Variable.from_dict(response.json())

    def create(self, project_suuid: str, name: str, value: str, is_masked: bool = False) -> Variable:
        """Create a new variable for a project

        Args:
            project_suuid (str): SUUID of the project
            name (str): Name of the variable
            value (str): Value of the variable
            is_masked (bool, optional): Whether the value should be masked or not. Defaults to False.

        Raises:
            PostError: Error based on response status code with the error message from the API

        Returns:
            Variable: A variable dataclass of the newly created variable
        """
        assert isinstance(is_masked, bool), "is_masked must be a boolean"

        response = client.create(
            url=client.askanna_url.variable.variable(),
            json={
                "name": name,
                "value": value,
                "is_masked": is_masked,
                "project_suuid": project_suuid,
            },
        )

        if response.status_code != 201:
            raise PostError(
                f"{response.status_code} - Something went wrong while creating the variable: {response.json()}"
            )

        return Variable.from_dict(response.json())

    def change(
        self,
        variable_suuid: str,
        name: Optional[str] = None,
        value: Optional[str] = None,
        is_masked: Optional[bool] = None,
    ) -> Variable:
        """Change the name, value or is_masked of a variable

        Note: is_masked can only be changed to True, a masked variable cannot be set to unmasked.

        Args:
            variable_suuid (str): SUUID of the variable to change
            name (str, optional): New name of the variable. Defaults to None.
            value (str, optional): New value of the variable. Defaults to None.
            is_masked (bool, optional): New is_masked value of the variable. Defaults to None.

        Raises:
            ValueError: Error if none of the arguments 'name', 'value' or 'is_masked' are provided.
            PatchError: Error based on response status code with the error message from the API

        Returns:
            Variable: The updated variable in a Variable dataclass
        """
        changes = {}
        if name:
            changes.update({"name": name})
        if value:
            changes.update({"value": value})
        if is_masked:
            assert isinstance(is_masked, bool), "is_masked must be a boolean"
            changes.update({"is_masked": is_masked})

        if not changes:
            raise ValueError("At least one of the arguments 'name', 'value' or 'is_masked' should be provided.")

        response = client.patch(
            url=client.askanna_url.variable.variable_detail(variable_suuid),
            json=changes,
        )

        if response.status_code == 404:
            raise PatchError(f"{response.status_code} - The variable SUUID '{variable_suuid}' was not found")
        if response.status_code != 200:
            raise PatchError(
                f"{response.status_code} - Something went wrong while updating the variable SUUID '{variable_suuid}': "
                f"{response.json()}"
            )

        return Variable.from_dict(response.json())

    def delete(self, variable_suuid: str) -> bool:
        """Delete a variable

        Args:
            variable_suuid (str): SUUID of the variable

        Raises:
            DeleteError: Error based on response status code with the error message from the API

        Returns:
            bool: True if the variable was succesfully deleted
        """
        response = client.delete(
            url=client.askanna_url.variable.variable_detail(variable_suuid),
        )

        if response.status_code == 404:
            raise DeleteError(f"{response.status_code} - The variable SUUID '{variable_suuid}' was not found")
        if response.status_code != 204:
            raise DeleteError(
                f"{response.status_code} - Something went wrong while deleting the variable SUUID '{variable_suuid}': "
                f"{response.json()}"
            )

        return True



import re
from dataclasses import dataclass
from typing import Dict

from askanna.config.utils import read_config, scan_config_in_path


@dataclass
class PushTarget:
    url: str = ""
    http_scheme: str = ""
    host: str = ""
    project_suuid: str = ""
    workspace_suuid: str = ""


@dataclass
class ProjectConfig:
    """Project configuration specifiction"""

    config_dict: Dict
    project_config_path: str
    push_target: PushTarget
    project_suuid: str
    workspace_suuid: str

    def reload_config(self, project_config_path: str = ""):
        new_config = load_config(project_config_path)

        self.config_dict = new_config.config_dict
        self.project_config_path = new_config.project_config_path
        self.push_target = new_config.push_target
        self.project_suuid = new_config.project_suuid
        self.workspace_suuid = new_config.workspace_suuid

    def clean_config(self):
        self.config_dict = {}
        self.project_config_path = ""
        self.push_target = PushTarget()
        self.project_suuid = ""
        self.workspace_suuid = ""


def extract_push_target(push_target: str) -> PushTarget:
    """
    Extract push target from the url configured
    Workspace is optional
    """
    if not push_target:
        raise ValueError("Cannot extract push-target if push-target is not set.")
    match_pattern = re.compile(
        r"(?P<http_scheme>https|http):\/\/(?P<askanna_host>[\w\.\-\:]+)\/(?P<workspace_suuid>[\w-]+){0,1}\/{0,1}project\/(?P<project_suuid>[\w-]+)\/{0,1}"  # noqa: E501
    )
    matches = match_pattern.match(push_target)

    if not matches:
        raise ValueError(
            f"The push target '{push_target}' is not valid. Please check the documentation to read more about the "
            "push target: https://docs.askanna.io/code/#push-target"
        )

    matches_dict = matches.groupdict()

    return PushTarget(
        url=push_target,
        http_scheme=matches_dict["http_scheme"],
        host=matches_dict["askanna_host"],
        workspace_suuid=matches_dict["workspace_suuid"],
        project_suuid=matches_dict["project_suuid"],
    )


def load_config(project_config_path: str = "") -> ProjectConfig:
    if not project_config_path:
        project_config_path = scan_config_in_path()

    project_config = {}
    push_target = PushTarget()
    workspace_suuid = ""
    project_suuid = ""

    if project_config_path:
        project_config = read_config(project_config_path)

    push_target_str = project_config.get("push-target")
    if push_target_str:
        push_target = extract_push_target(push_target_str)
        workspace_suuid = push_target.workspace_suuid
        project_suuid = push_target.project_suuid

    return ProjectConfig(
        config_dict=project_config,
        project_config_path=project_config_path,
        push_target=push_target,
        workspace_suuid=workspace_suuid,
        project_suuid=project_suuid,
    )


config: ProjectConfig = load_config()



import os
import sys
from dataclasses import dataclass
from typing import Dict, Optional

import click

from askanna.config.utils import read_config, store_config
from askanna.settings import (
    DEFAULT_SERVER_CONFIG_PATH,
    DEFAULT_SERVER_REMOTE,
    DEFAULT_SERVER_UI,
)

SERVER_CONFIG_PATH = os.getenv("AA_SERVER_CONFIG_FILE", DEFAULT_SERVER_CONFIG_PATH)
SERVER = os.getenv("AA_SERVER", "default")
DEFAULT_SERVER_CONFIG = {
    "server": {
        "default": {
            "remote": f"{DEFAULT_SERVER_REMOTE}",
            "ui": f"{DEFAULT_SERVER_UI}",
            "token": "",
        },
    }
}


@dataclass
class ServerConfig:
    """Server configuration specification"""

    remote: str
    config_dict: Optional[Dict] = None
    server_config_path: str = SERVER_CONFIG_PATH
    server: str = "default"
    token: str = ""
    ui: str = ""

    @property
    def is_authenticated(self) -> bool:
        if self.token:
            return True
        return False

    def save_server_to_config_file(self):
        self.config_dict = add_or_update_server_in_config_dict(
            self.config_dict, self.server, self.remote, self.token, self.ui
        )
        store_config(self.server_config_path, self.config_dict)

    def logout_and_remove_token(self):
        self.token = ""  # nosec
        self.save_server_to_config_file()


def add_or_update_server_in_config_dict(  # nosec
    config_dict: Optional[Dict], server: str, remote: str, token: str = "", ui: str = ""
) -> Dict:
    server_dict = {
        "remote": remote,
        "ui": ui,
        "token": token,
    }

    if config_dict and config_dict.get("server", {}).get(server):
        config_dict["server"][server].update(server_dict)
    elif config_dict and config_dict.get("server"):
        config_dict["server"].update({server: server_dict})
    else:
        config_dict = {}
        config_dict["server"] = {server: server_dict}

    return config_dict


def load_config(server_config_path: str = SERVER_CONFIG_PATH, server: str = SERVER) -> ServerConfig:
    config_dict = DEFAULT_SERVER_CONFIG

    if os.path.isfile(server_config_path):
        local_config = read_config(server_config_path)
        config_dict.update(local_config)

    server_dict = config_dict.get("server", {}).get(server, {})
    if not server_dict and server != "default":
        click.echo(
            f"Server settings for '{server}' not found. Please try again with a valid server, or add this " "server.",
            err=True,
        )
        sys.exit(1)

    remote = os.getenv("AA_REMOTE", server_dict.get("remote", DEFAULT_SERVER_REMOTE))
    ui = os.getenv("AA_UI", server_dict.get("ui", DEFAULT_SERVER_UI))
    token = os.getenv("AA_TOKEN", server_dict.get("token", ""))

    return ServerConfig(
        config_dict=config_dict,
        server_config_path=server_config_path,
        server=server,
        remote=remote,
        ui=ui,
        token=token,
    )


config: ServerConfig = load_config()



from typing import List, Optional

from askanna.core.dataclasses.job import Job
from askanna.core.dataclasses.run import RunStatus
from askanna.core.exceptions import DeleteError, GetError, PatchError, PostError
from askanna.gateways.api_client import client

from .utils import ListResponse


class JobListResponse(ListResponse):
    def __init__(self, data: dict):
        super().__init__(data)
        self.results: List[Job] = [Job.from_dict(job) for job in data["results"]]

    @property
    def jobs(self):
        return self.results


class JobGateway:
    """Management of jobs in AskAnna
    This is the class which act as gateway to the API of AskAnna
    """

    def list(
        self,
        project_suuid: Optional[str] = None,
        workspace_suuid: Optional[str] = None,
        page_size: Optional[int] = None,
        cursor: Optional[str] = None,
        order_by: Optional[str] = None,
        search: Optional[str] = None,
    ) -> JobListResponse:
        """List all jobs with filter and order options

        Args:
            project_suuid (str, optional): Project SUUID to filter for jobs in a project. Defaults to None.
            workspace_suuid (str, optional): Workspace SUUID to filter for jobs in a workspace. Defaults to None.
            page_size (int, optional): Number of jobs to return per page. Defaults to the default value of
                the backend.
            cursor (str, optional): Cursor to start the page from. Defaults to None.
            order_by (str, optional): Order by field(s). Defaults to the default value of the backend.
            search (str, optional): Search for a specific job.

        Raises:
            GetError: Error based on response status code with the error message from the API

        Returns:
            JobListResponse: The response from the API with a list of jobs and pagination information
        """
        assert page_size is None or page_size > 0, "page_size must be a positive integer"

        response = client.get(
            url=client.askanna_url.job.job_list(),
            params={
                "project_suuid": project_suuid,
                "workspace_suuid": workspace_suuid,
                "page_size": page_size,
                "cursor": cursor,
                "order_by": order_by,
                "search": search,
            },
        )

        if response.status_code != 200:
            error_message = f"{response.status_code} - Something went wrong while retrieving the job list"
            try:
                error_message += f":\n  {response.json()}"
            except ValueError:
                pass
            raise GetError(error_message)

        return JobListResponse(response.json())

    def detail(self, job_suuid: str) -> Job:
        """Get information of a job

        Args:
            job_suuid (str): SUUID of the job

        Raises:
            GetError: Error based on response status code with the error message from the API

        Returns:
            Job: Job information in a Job dataclass
        """

        response = client.get(
            url=client.askanna_url.job.job_detail(job_suuid),
        )

        if response.status_code == 404:
            raise GetError(f"404 - The job SUUID '{job_suuid}' was not found")
        if response.status_code != 200:
            raise GetError(
                f"{response.status_code} - Something went wrong while retrieving job SUUID '{job_suuid}': "
                f"{response.json()}"
            )

        return Job.from_dict(response.json())

    def run_request(
        self,
        job_suuid: str,
        data: Optional[dict] = None,
        name: Optional[str] = None,
        description: Optional[str] = None,
    ) -> RunStatus:
        """Do a request to run a job on the AskAnna platform

        Args:
            job_suuid (str): SUUID of the job you want to start a run of

        Raises:
            PostError: Error based on response status code with the error message from the API

        Returns:
            RunStatus: The run status information in a RunStatus dataclass
        """
        response = client.post(
            url=client.askanna_url.job.run_request(job_suuid),
            json=data,
            params={
                "name": name,
                "description": description,
            },
        )

        if response.status_code == 404:
            raise PostError(f"404 - The job SUUID '{job_suuid}' was not found")
        if response.status_code != 201:
            raise PostError(
                f"{response.status_code} - Something went wrong while starting the run for job SUUID '{job_suuid}': "
                f"{response.json()}"
            )

        return RunStatus.from_dict(response.json())

    def change(
        self,
        job_suuid: str,
        name: Optional[str] = None,
        description: Optional[str] = None,
    ) -> Job:
        """Change the name and/or description of a job

        Args:
            job_suuid (str): SUUID of the job you want to change the information of
            name (str, optional): New name for the job. Defaults to None.
            description (str, optional): New description of the job. Defaults to None.

        Raises:
            ValueError: Error if none of the arguments 'name' or 'description' are provided
            PatchError: Error based on response status code with the error message from the API

        Returns:
            Job: The changed job information in a Job dataclass
        """
        changes = {}
        if name:
            changes.update({"name": name})
        if description:
            changes.update({"description": description})

        if not changes:
            raise ValueError("At least one of the parameters 'name' or 'description' must be set")

        response = client.patch(url=client.askanna_url.job.job_detail(job_suuid), json=changes)

        if response.status_code == 404:
            raise PatchError(f"404 - The job SUUID '{job_suuid}' was not found")
        if response.status_code != 200:
            raise PatchError(
                f"{response.status_code} - Something went wrong while updating the job SUUID '{job_suuid}': "
                f"{response.json()}"
            )

        return Job.from_dict(response.json())

    def delete(self, job_suuid: str) -> bool:
        """Delete a job

        Args:
            job_suuid (str): SUUID of the job you want to delete

        Raises:
            DeleteError: Error based on response status code with the error message from the API

        Returns:
            bool: True if the job was succesfully deleted
        """
        response = client.delete(
            url=client.askanna_url.job.job_detail(job_suuid),
        )

        if response.status_code == 404:
            raise DeleteError(f"404 - The job SUUID '{job_suuid}' was not found")
        if response.status_code != 204:
            raise DeleteError(
                f"{response.status_code} - Something went wrong while deleting the job SUUID '{job_suuid}': "
                f"{response.json()}"
            )

        return True



import sys

import click

from askanna.cli.utils import ask_which_workspace
from askanna.config import config
from askanna.config.utils import string_format_datetime
from askanna.core.exceptions import GetError, PatchError
from askanna.sdk.workspace import WorkspaceSDK


@click.group()
def cli1():
    pass


@cli1.command(help="List workspaces available in AskAnna", short_help="List workspaces")
@click.option("--search", "-s", required=False, type=str, help="Search for a specific workspace")
@click.option(
    "--member/--no-member",
    "-m",
    "is_member",
    default=None,
    required=False,
    type=bool,
    help="Filter on workspaces where the authenticated user is a member.",
)
@click.option(
    "--visibility", "-v", required=False, type=str, help="Filter on workspaces with visibility private or public"
)
def list(search, is_member, visibility):
    workspace_sdk = WorkspaceSDK()
    try:
        workspaces = workspace_sdk.list(
            number_of_results=100,
            search=search,
            is_member=is_member,
            visibility=visibility,
            order_by="name",
        )
    except Exception as e:
        click.echo(f"Something went wrong while listing the workspaces:\n  {e}", err=True)
        sys.exit(1)

    if not workspaces:
        click.echo("We cannot find any workspaces.")
        sys.exit(0)

    click.echo("")
    click.echo("-------------------    -------------------------")
    click.echo("WORKSPACE SUUID        WORKSPACE NAME")
    click.echo("-------------------    -------------------------")

    for workspace in workspaces:
        workspace_name = f"{workspace.name[:22]}..." if len(workspace.name) > 25 else workspace.name[:25]
        click.echo(f"{workspace.suuid}    {workspace_name}")

    if len(workspaces) != workspace_sdk.list_total_count:
        click.echo("")
        click.echo(f"Note: the first {len(workspaces):,} of {workspace_sdk.list_total_count:,} workspaces are shown.")

    click.echo("")


@cli1.command(help="Get information about a workspace", short_help="Get workspace info")
@click.option("--id", "-i", "workspace_suuid", required=False, type=str, help="Workspace SUUID")
def info(workspace_suuid):
    if not workspace_suuid:
        workspace_suuid = config.project.workspace_suuid

    if workspace_suuid:
        try:
            workspace = WorkspaceSDK().get(workspace_suuid)
        except GetError as e:
            click.echo(f"Something went wrong while getting the workspace:\n  {e}", err=True)
            sys.exit(1)
    else:
        workspace = ask_which_workspace(question="Which workspace do you want to get?")

    click.echo("")
    click.echo(f"Name:        {workspace.name}")
    click.echo(f"SUUID:       {workspace.suuid}")
    click.echo(f"Description: {workspace.description}")
    click.echo(f"Visibility:  {workspace.visibility}")
    click.echo("")
    click.echo(f"Created:     {workspace.created_at.strftime(string_format_datetime)}")
    click.echo(f"Modified:    {workspace.modified_at.strftime(string_format_datetime)}")
    click.echo("")


@cli1.command(help="Change workspace information in AskAnna", short_help="Change workspace")
@click.option("--id", "-i", "workspace_suuid", required=False, type=str, help="Workspace SUUID")
@click.option("--name", "-n", required=False, type=str, help="New name to set")
@click.option("--description", "-d", required=False, type=str, help="New description to set")
@click.option("--visibility", "-v", required=False, type=str, help="New visibility to set (PRIVATE or PUBLIC)")
def change(workspace_suuid, name, description, visibility):
    if not workspace_suuid:
        workspace_suuid = config.project.workspace_suuid
        if not workspace_suuid:
            workspace = ask_which_workspace(question="Which workspace do you want to change?")
            workspace_suuid = workspace.suuid

    if not name and not description and not visibility:
        if click.confirm("\nDo you want to change the name of the workspace?"):
            name = click.prompt("New name of the workspace", type=str)
        if click.confirm("\nDo you want to change the description of the workspace?"):
            description = click.prompt("New description of the workspace", type=str)
        if click.confirm("\nDo you want to change the visibility of the workspace?"):
            visibility = click.prompt("Visibility of the workspace (PRIVATE or PUBLIC)", type=str)

        click.confirm("\nDo you want to change the workspace?", abort=True)

    try:
        workspace = WorkspaceSDK().change(
            workspace_suuid=workspace_suuid, name=name, description=description, visibility=visibility
        )
    except PatchError as e:
        if str(e).startswith("404"):
            click.echo(f"The workspace SUUID '{workspace_suuid}' was not found", err=True)
            sys.exit(1)
        else:
            click.echo(
                f"Something went wrong while changing the workspace SUUID '{workspace_suuid}':\n  {e}", err=True
            )
            sys.exit(1)
    else:
        click.echo(f"\nYou succesfully changed workspace '{workspace.name}' with SUUID '{workspace.suuid}'")


@cli1.command(help="Create a new workspace in AskAnna", short_help="Create workspace")
@click.option("--name", "-n", required=False, type=str, help="Name of the workspace")
@click.option("--description", "-d", required=False, type=str, help="Description of the workspace")
@click.option(
    "--visibility",
    "-v",
    required=False,
    type=str,
    default="PRIVATE",
    help="Workspace visibility (PRIVATE (default) or PUBLIC)",
)
def create(name, description, visibility):
    if not name:
        name = click.prompt("\nName of the workspace", type=str)

        if not description:
            description = click.prompt(
                "Optional description of the workspace", type=str, default="", show_default=False
            )

    click.confirm(f'\nDo you want to create the workspace "{name}"?', abort=True)

    try:
        workspace = WorkspaceSDK().create(name=name, description=description, visibility=visibility)
    except Exception as e:
        click.echo(f"Something went wrong while creating the workspace:\n  {e}", err=True)
        sys.exit(1)
    else:
        click.echo(f"\nYou successfully created workspace '{workspace.name}' with SUUID '{workspace.suuid}'")


@cli1.command(help="Remove a workspace in AskAnna", short_help="Remove workspace")
@click.option(
    "--id", "-i", "workspace_suuid", type=str, required=True, help="Workspace SUUID", prompt="Workspace SUUID"
)
@click.option("--force", "-f", is_flag=True, help="Force")
def remove(workspace_suuid, force):
    try:
        workspace = WorkspaceSDK().get(workspace_suuid)
    except GetError as e:
        if str(e).startswith("404"):
            click.echo(f"The workspace SUUID '{workspace_suuid}' was not found", err=True)
            sys.exit(1)
        else:
            click.echo(
                f"Something went wrong while removing the workspace SUUID '{workspace_suuid}':\n  {e}", err=True
            )
            sys.exit(1)

    if not force:
        if not click.confirm(
            f"Are you sure to remove workspace SUUID '{workspace_suuid}' with name '{workspace.name}'?"
        ):
            click.echo("Understood. We are not removing the workspace.")
            sys.exit(0)

    try:
        removed = WorkspaceSDK().delete(workspace_suuid)
    except Exception as e:
        click.echo(f"Something went wrong while removing the workspace SUUID '{workspace_suuid}':\n  {e}", err=True)
        sys.exit(1)
    else:
        if removed:
            click.echo(f"You removed workspace SUUID '{workspace_suuid}'")
        else:
            click.echo(f"Something went wrong. Removing the workspace SUUID '{workspace_suuid}' aborted.", err=True)
            sys.exit(1)


cli = click.CommandCollection(
    sources=[cli1],
    help="Manage your workspaces in AskAnna",
    short_help="Manage workspaces in AskAnna",
)



import importlib

import click

from askanna import ASKANNA_VERSION

HELP = """
The run util is used to support AskAnna runs
"""

SHORT_HELP = "AskAnna Run Utils"

EPILOG = """
For usage and help on a specific command, run it with a --help flag, e.g.:

    askanna-run-utils [COMMAND] --help
"""


@click.group(
    help=HELP,
    short_help=SHORT_HELP,
    epilog=EPILOG,
)
@click.version_option(version=ASKANNA_VERSION, prog_name="AskAnna Run Utils")
def cli():
    """
    Initialize the AskAnna Run Utils commands
    """


commands = [
    "push_artifact",
    "push_metrics",
    "push_result",
    "push_variables",
    "get_package",
    "get_payload",
    "get_run_manifest",
]

for command in commands:
    module_path = "askanna.cli.run_utils." + command
    command_module = importlib.import_module(module_path)
    command_name = command.replace("_", "-")
    cli.add_command(command_module.cli, command_name)



import sys

import click

from askanna.cli.utils import (
    ask_which_job,
    ask_which_project,
    ask_which_workspace,
    job_run_request,
)
from askanna.config import config
from askanna.config.utils import string_format_datetime
from askanna.core.exceptions import GetError, PatchError
from askanna.sdk.job import JobSDK


@click.group()
def cli1():
    pass


@cli1.command(help="List jobs available in AskAnna", short_help="List jobs")
@click.option(
    "--project",
    "-p",
    "project_suuid",
    required=False,
    type=str,
    help="Project SUUID to list jobs for a project",
)
@click.option(
    "--workspace",
    "-w",
    "workspace_suuid",
    required=False,
    type=str,
    help="Workspace SUUID to list jobs for a workspace",
)
@click.option("--search", "-s", required=False, type=str, help="Search for a specific job")
def list(project_suuid, workspace_suuid, search):
    job_sdk = JobSDK()
    try:
        jobs = job_sdk.list(
            number_of_results=100,
            project_suuid=project_suuid,
            workspace_suuid=workspace_suuid,
            search=search,
            order_by="project.name,name",
        )
    except Exception as e:
        click.echo(f"Something went wrong while listing the jobs:\n  {e}", err=True)
        sys.exit(1)

    if not jobs:
        click.echo("We cannot find any job.")
        sys.exit(0)

    if project_suuid:
        click.echo(f"The jobs for project '{jobs[0].project.name}' are:\n")
        click.echo("")
        click.echo("-------------------    -------------------------")
        click.echo("JOB SUUID              JOB NAME")
        click.echo("-------------------    -------------------------")
    if not project_suuid and workspace_suuid:
        click.echo(f"The jobs for workspace '{jobs[0].workspace.name}' are:")
    if not project_suuid:
        click.echo("")
        click.echo("-------------------    --------------------    -------------------    -------------------------")
        click.echo("PROJECT SUUID          PROJECT NAME            JOB SUUID              JOB NAME")
        click.echo("-------------------    --------------------    -------------------    -------------------------")

    for job in jobs:
        job_name = f"{job.name[:22]}..." if len(job.name) > 25 else job.name[:25]
        if project_suuid:
            click.echo(f"{job.suuid}    {job_name}")
        else:
            project_name = f"{job.project.name[:17]}..." if len(job.project.name) > 20 else job.project.name[:20]
            click.echo(
                "{project_suuid}    {project_name}    {job_suuid}    {job_name}".format(
                    project_suuid=job.project.suuid,
                    project_name=f"{project_name:20}",
                    job_suuid=job.suuid,
                    job_name=job_name,
                )
            )

    if len(jobs) != job_sdk.list_total_count:
        click.echo("")
        click.echo(f"Note: the first {len(jobs):,} of {job_sdk.list_total_count:,} jobs are shown.")

    click.echo("")


@cli1.command(help="Get information about a job", short_help="Get job info")
@click.option("--id", "-i", "job_suuid", required=False, type=str, help="Job SUUID")
def info(job_suuid):
    if job_suuid:
        try:
            job = JobSDK().get(job_suuid=job_suuid)
        except Exception as e:
            click.echo(f"Something went wrong while getting the job info:\n  {e}", err=True)
            sys.exit(1)
    else:
        project_suuid = config.project.project_suuid
        if not project_suuid:
            workspace = ask_which_workspace(question="From which workspace do you want to get a job?")
            project = ask_which_project(
                question="From which project do you want to get a job?", workspace_suuid=workspace.suuid
            )
            project_suuid = project.suuid

        job = ask_which_job(question="Which job do you want to get?", project_suuid=project_suuid)

    scheduled = "Yes" if job.schedules else "No"
    notifications = "No"
    if job.notifications and (
        job.notifications.get("all", {}).get("email") or job.notifications.get("error", {}).get("email")
    ):
        notifications = "Yes"

    click.echo("")
    click.echo(f"Name:            {job.name}")
    click.echo(f"SUUID:           {job.suuid}")
    click.echo(f"Description:     {job.description}")
    click.echo("")
    click.echo(f"Environment:     {job.environment}")
    click.echo(f"Timezone:        {job.timezone}")
    click.echo(f"Scheduled:       {scheduled}")
    click.echo(f"Notifications:   {notifications}")
    click.echo("")
    click.echo(f"Project:         {job.project.name}")
    click.echo(f"Project SUUID:   {job.project.suuid}")
    click.echo(f"Workspace:       {job.workspace.name}")
    click.echo(f"Workspace SUUID: {job.workspace.suuid}")
    click.echo("")
    click.echo(f"Created:         {job.created_at.strftime(string_format_datetime)}")
    click.echo(f"Modified:        {job.modified_at.strftime(string_format_datetime)}")
    click.echo("")


@cli1.command(help="Do a request to run a job on the AskAnna platform", short_help="Request to run a job")
@click.argument("job_name", required=False, type=str)
@click.option("--id", "-i", "job_suuid", required=False, type=str, help="SUUID of the job to run")
@click.option("--data", "-d", required=False, type=str, default=None, help="JSON data")
@click.option(
    "--data-file",
    "-D",
    "data_file",
    required=False,
    type=str,
    default=None,
    help="File with JSON data",
)
@click.option(
    "--push/--no-push",
    "-p",
    "push_code",
    default=False,
    show_default=True,
    help="Push code before starting a run",
)
@click.option("--name", "-n", required=False, type=str, help="Give the run a name")
@click.option(
    "--description",
    required=False,
    type=str,
    help="Description of the run",
    default=None,
)
@click.option("--project", "project_suuid", required=False, type=str, help="Project SUUID")
@click.option("--workspace", "workspace_suuid", required=False, type=str, help="Workspace SUUID")
def run_request(
    job_name,
    job_suuid,
    name,
    description,
    data,
    data_file,
    push_code,
    project_suuid,
    workspace_suuid,
):
    job_run_request(
        job_name=job_name,
        job_suuid=job_suuid,
        name=name,
        description=description,
        data=data,
        data_file=data_file,
        push_code=push_code,
        project_suuid=project_suuid,
        workspace_suuid=workspace_suuid,
    )


@cli1.command(help="Change job information in AskAnna", short_help="Change job")
@click.option("--id", "-i", "job_suuid", required=False, type=str, help="Job SUUID")
@click.option("--name", "-n", required=False, type=str, help="New name to set")
@click.option("--description", "-d", required=False, type=str, help="New description to set")
def change(job_suuid, name, description):
    if not job_suuid:
        project_suuid = config.project.project_suuid

        if not project_suuid:
            workspace = ask_which_workspace(question="From which workspace do you want to change a job?")
            project = ask_which_project(
                question="From which project do you want to change a job?", workspace_suuid=workspace.suuid
            )
            project_suuid = project.suuid

        job = ask_which_job(question="Which job do you want to change?", project_suuid=project_suuid)
        job_suuid = job.suuid

    if not name and not description:
        if click.confirm("\nDo you want to change the name of the job?"):
            name = click.prompt("New name of the job", type=str)
        if click.confirm("\nDo you want to change the description of the job?"):
            description = click.prompt("New description of the job", type=str)

        click.confirm("\nDo you want to change the job?", abort=True)

    try:
        job = JobSDK().change(job_suuid=job_suuid, name=name, description=description)
    except PatchError as e:
        if str(e).startswith("404"):
            click.echo(f"The job SUUID '{job_suuid}' was not found", err=True)
            sys.exit(1)
        else:
            click.echo(f"Something went wrong while changing the job SUUID '{job_suuid}':\n  {e}", err=True)
            sys.exit(1)
    else:
        click.echo(f"\nYou succesfully changed job '{job.name}' with SUUID '{job.suuid}'")


@cli1.command(help="Remove a job in AskAnna", short_help="Remove job")
@click.option("--id", "-i", "job_suuid", type=str, required=True, help="Job SUUID", prompt="Job SUUID")
@click.option("--force", "-f", is_flag=True, help="Force")
def remove(job_suuid, force):
    try:
        job = JobSDK().get(job_suuid=job_suuid)
    except GetError as e:
        if str(e).startswith("404"):
            click.echo(f"The job SUUID '{job_suuid}' was not found", err=True)
            sys.exit(1)
        else:
            click.echo(f"Something went wrong while removing the job SUUID '{job_suuid}':\n  {e}", err=True)
            sys.exit(1)

    if not force:
        if not click.confirm(f"Are you sure to remove job SUUID '{job_suuid}' with name '{job.name}'?"):
            click.echo("Understood. We are not removing the job.")
            sys.exit(0)

    try:
        _ = JobSDK().delete(job_suuid=job_suuid)
    except Exception as e:
        click.echo(f"Something went wrong while removing the job SUUID '{job_suuid}':\n  {e}", err=True)
        sys.exit(1)
    else:
        click.echo(f"You removed job SUUID '{job_suuid}'")


cli = click.CommandCollection(
    sources=[cli1],
    help="Manage your jobs in AskAnna",
    short_help="Manage jobs in AskAnna",
)



import click

from askanna.sdk.track import variable_collector

HELP = """
Retrieve variables generated with track_variable(s) and push variables to AskAnna
"""

SHORT_HELP = "Push variables to AskAnna"


@click.command(help=HELP, short_help=SHORT_HELP)
@click.option("--force", "-f", is_flag=True, help="Force push variables")
def cli(force):
    if len(variable_collector) > 0 or force:
        variable_collector.save(force=True)



import json
import sys
from typing import List, Optional, Union

import click

from askanna.config import config
from askanna.core.dataclasses.job import Job
from askanna.core.dataclasses.project import Project
from askanna.core.dataclasses.run import Run
from askanna.core.dataclasses.variable import Variable
from askanna.core.dataclasses.workspace import Workspace
from askanna.core.push import push
from askanna.sdk.job import JobSDK
from askanna.sdk.project import ProjectSDK
from askanna.sdk.run import RunSDK
from askanna.sdk.variable import VariableSDK
from askanna.sdk.workspace import WorkspaceSDK


def make_option_list_string_for_question(
    list_items: Union[List[Workspace], List[Project], List[Job], List[Run], List[Variable]],
    name_with_suuid: bool = False,
) -> str:
    """From a list of instance items, create a string with the name and optionally suuid of each instance.

    Args:
        list_items (list): a list of intance items. Instance items can be Workspace, Project, Job, Run or Variable.
        name_with_suuid (bool, optional): Boolean if the SUUID should be included in the string. Defaults to False.

    Returns:
        str: a string with the name and optionally suuid of each instance where each instance is on a new line as a
             numbered list.
    """
    list_str = ""
    for idx, item in enumerate(list_items, start=1):
        if name_with_suuid:
            name = f"{item.name} ({item.suuid})" if item.name else item.suuid
        else:
            name = item.name if item.name else item.suuid
        if idx < 10 and len(list_items) >= 10:
            list_str += "%d.  %s\n" % (idx, name)
        else:
            list_str += "%d. %s\n" % (idx, name)

    return list_str


def ask_which_workspace(question: Optional[str] = None) -> Workspace:
    """
    Determine which workspace should be used to perform an action
    """
    workspace_sdk = WorkspaceSDK()
    workspace_list = workspace_sdk.list(number_of_results=99)

    if len(workspace_list) == 0:
        click.echo("This request doesn't return any workspaces. Did you create a workspace?", err=True)
        sys.exit(0)
    elif len(workspace_list) == 1:
        workspace = workspace_list[0]
    else:
        question = question or "Which workspace do you want to select?"
        list_str = make_option_list_string_for_question(workspace_list)

        if workspace_sdk.list_total_count and workspace_sdk.list_total_count > len(workspace_list):
            list_str += (
                f"\nNote: the {len(workspace_list):,} most recent workspaces of {workspace_sdk.list_total_count:,} "
                "workspaces are in the list.\n"
            )

        selected_workspace = click.prompt(
            "\n" + question + "\n\n" + list_str + "\n" + "Please enter the workspace number",
            type=click.Choice([str(i + 1) for i in range(len(workspace_list))]),
            show_choices=False,
        )

        workspace = workspace_list[int(selected_workspace) - 1]

    click.echo(f"Selected workspace: {workspace.name}")
    return workspace


def ask_which_project(question: Optional[str] = None, workspace_suuid: Optional[str] = None) -> Project:
    """
    Determine which project should be used to perform an action
    """
    project_sdk = ProjectSDK()
    project_list = project_sdk.list(workspace_suuid=workspace_suuid, number_of_results=99)

    if len(project_list) == 0:
        click.echo("This request doesn't return any projects. Did you create a project?", err=True)
        sys.exit(0)
    elif len(project_list) == 1:
        project = project_list[0]
    else:
        question = question or "Which project do you want to select?"
        list_str = make_option_list_string_for_question(project_list)

        if project_sdk.list_total_count and project_sdk.list_total_count > len(project_list):
            list_str += (
                f"\nNote: the {len(project_list):,} most recent projects of {project_sdk.list_total_count:,} "
                "projects are in the list.\n"
            )

        selected_project = click.prompt(
            "\n" + question + "\n\n" + list_str + "\n" + "Please enter the project number",
            type=click.Choice([str(i + 1) for i in range(len(project_list))]),
            show_choices=False,
        )

        project = project_list[int(selected_project) - 1]

    click.echo(f"Selected project: {project.name}")
    return project


def ask_which_job(question: Optional[str] = None, project_suuid: Optional[str] = None) -> Job:
    """
    Determine which job should be used to perform an action
    """
    job_sdk = JobSDK()
    job_list = job_sdk.list(project_suuid=project_suuid, number_of_results=99)

    if len(job_list) == 0:
        click.echo(
            "This request doesn't return any jobs. Did you push code with an askanna.yml config file?", err=True
        )
        sys.exit(0)
    elif len(job_list) == 1:
        job = job_list[0]
    else:
        question = question or "Which job do you want to select?"
        list_str = make_option_list_string_for_question(job_list)

        if job_sdk.list_total_count and job_sdk.list_total_count > len(job_list):
            list_str += (
                f"\nNote: the {len(job_list):,} most recent jobs of {job_sdk.list_total_count:,} jobs are in the "
                "list.\n"
            )

        selected_job = click.prompt(
            "\n" + question + "\n\n" + list_str + "\n" + "Please enter the job number",
            type=click.Choice([str(i + 1) for i in range(len(job_list))]),
            show_choices=False,
        )

        job = job_list[int(selected_job) - 1]

    click.echo(f"Selected job: {job.name}")
    return job


def ask_which_run(question: Optional[str] = None, job_suuid: Optional[str] = None) -> Run:
    """
    Determine which job should be used to perform an action
    """
    run_sdk = RunSDK()
    run_list = run_sdk.list(job_suuid=job_suuid, number_of_results=99)

    if len(run_list) == 0:
        click.echo("This request doesn't return any runs. Did you start a run?")
        sys.exit(0)
    elif len(run_list) == 1:
        run = run_list[0]
    else:
        question = question or "Which run do you want to select?"
        list_str = make_option_list_string_for_question(run_list, name_with_suuid=True)

        if run_sdk.list_total_count and run_sdk.list_total_count > len(run_list):
            list_str += (
                f"\nNote: the {len(run_list):,} most recent runs of {run_sdk.list_total_count:,} runs are in the "
                "list.\n"
            )

        selected_run = click.prompt(
            "\n" + question + "\n\n" + list_str + "\n" + "Please enter the run number",
            type=click.Choice([str(i + 1) for i in range(len(run_list))]),
            show_choices=False,
        )

        run = run_list[int(selected_run) - 1]

    click.echo(f"Selected run: {run.suuid}")
    return run


def ask_which_variable(question: Optional[str] = None, project_suuid: Optional[str] = None) -> Variable:
    """
    Determine which variable should be used to perform an action
    """
    variable_sdk = VariableSDK()
    variable_list = variable_sdk.list(project_suuid=project_suuid, number_of_results=99)

    if len(variable_list) == 0:
        click.echo("This request doesn't return any variables. Did you create a variable?", err=True)
        sys.exit(0)
    elif len(variable_list) == 1:
        variable = variable_list[0]
    else:
        question = question or "Which variable do you want to select?"
        list_str = make_option_list_string_for_question(variable_list)

        if variable_sdk.list_total_count and variable_sdk.list_total_count > len(variable_list):
            list_str += (
                f"\nNote: the {len(variable_list):,} most recent variables of {variable_sdk.list_total_count:,} "
                "variables are in the list.\n"
            )

        selected_variable = click.prompt(
            "\n" + question + "\n\n" + list_str + "\n" + "Please enter the variable number",
            type=click.Choice([str(i + 1) for i in range(len(variable_list))]),
            show_choices=False,
        )

        variable = variable_list[int(selected_variable) - 1]

    click.echo(f"Selected variable: {variable.name}")
    return variable


def determine_project_for_run_request(
    project_suuid: Optional[str] = None,
    workspace_suuid: Optional[str] = None,
) -> Project:
    if not project_suuid:
        project_suuid = config.project.project_suuid

    # Still if there is no project_suuid found, we will ask which project to use
    if project_suuid:
        project = ProjectSDK().get(project_suuid=project_suuid)
        click.echo(f"Selected project: {project.name}")
        return project
    else:
        if not workspace_suuid:
            workspace = ask_which_workspace(question="From which workspace do you want to run a job?")
            workspace_suuid = workspace.suuid

        return ask_which_project(
            question="From which project do you want to run a job?",
            workspace_suuid=workspace_suuid,
        )


def job_run_request(
    job_suuid: Optional[str] = None,
    job_name: Optional[str] = None,
    name: Optional[str] = None,
    description: Optional[str] = None,
    data: Optional[str] = None,
    data_file: Optional[str] = None,
    push_code: bool = False,
    project_suuid: Optional[str] = None,
    workspace_suuid: Optional[str] = None,
):
    json_data = None
    if data and data_file:
        click.echo("Cannot use both --data and --data-file.", err=True)
        sys.exit(1)
    elif data:
        json_data = json.loads(data)
    elif data_file:
        with open(data_file) as json_file:
            json_data = json.load(json_file)

    if json_data and not type(json_data) == dict:
        click.echo("The data cannot be processes becaused it's not in a JSON format.", err=True)
        sys.exit(1)

    if push_code:
        push(overwrite=True, description=description)

    if not job_suuid and not project_suuid:
        project = determine_project_for_run_request(project_suuid, workspace_suuid)
        project_suuid = project.suuid

    if job_suuid:
        pass
    elif job_name:
        job_name = job_name.strip()
        try:
            job = JobSDK().get_job_by_name(job_name=job_name, project_suuid=project_suuid)
            job_suuid = job.suuid
        except Exception as e:
            click.echo(e)
            sys.exit(1)
    else:
        job = ask_which_job(
            question="Which job do you want to run?",
            project_suuid=project_suuid,
        )

        if not click.confirm(f"\nDo you want to run the job '{job.name}'?", abort=False):
            click.echo(f"\nAborted! Not running job '{job.name}'.")
            sys.exit(0)
        else:
            click.echo("")

        job_suuid = job.suuid

    try:
        run = JobSDK().run_request(
            job_suuid=job_suuid,
            data=json_data,
            name=name,
            description=description,
        )
    except Exception as e:
        click.echo(e)
        sys.exit(1)
    else:
        click.echo(f"Succesfully started a new run for job '{run.job.name}' in AskAnna with SUUID " f"'{run.suuid}'.")



import json

import requests
from requests.structures import CaseInsensitiveDict

from askanna import ASKANNA_VERSION, USING_ASKANNA_CLI
from askanna.config import config
from askanna.config.api_url import askanna_url
from askanna.core.exceptions import ConnectionError
from askanna.core.utils.object import json_serializer


class Client:
    """
    Client for communication with AskAnna API service
    """

    def __init__(self):
        self.config = config
        self.askanna_url = askanna_url
        self.session = requests.Session()
        self.session.headers.update(self.generate_authenication_header())

    def generate_authenication_header(self) -> CaseInsensitiveDict:
        askanna_agent = USING_ASKANNA_CLI and "cli" or "python-sdk"
        auth_header = CaseInsensitiveDict(
            {
                "askanna-agent": askanna_agent,
                "askanna-agent-version": ASKANNA_VERSION,
                "user-agent": f"askanna-python/{ASKANNA_VERSION}",
            }
        )

        if self.config.server.is_authenticated:
            auth_header.update({"Authorization": f"Token {self.config.server.token}"})

        return auth_header

    def update_session(self):
        self.session.headers.update(self.generate_authenication_header())

    def _connection_error_message(self, url, error):
        connection_error_message_base = "Something went wrong. Please check whether the URL is an AskAnna Backend."
        return f"{connection_error_message_base}\n    URL:    {url}\n    Error: {error}"

    def get(self, url, **kwargs):
        try:
            return self.session.get(url, **kwargs)
        except requests.exceptions.ConnectionError as e:
            raise ConnectionError(self._connection_error_message(url, e))

    def head(self, url, **kwargs):
        try:
            return self.session.head(url, **kwargs)
        except requests.exceptions.ConnectionError as e:
            raise ConnectionError(self._connection_error_message(url, e))

    def patch(self, url, **kwargs):
        if kwargs.get("json"):
            kwargs["json"] = json.loads(json.dumps(kwargs.get("json"), default=json_serializer))

        try:
            return self.session.patch(url, **kwargs)
        except requests.exceptions.ConnectionError as e:
            raise ConnectionError(self._connection_error_message(url, e))

    def put(self, url, **kwargs):
        if kwargs.get("json"):
            kwargs["json"] = json.loads(json.dumps(kwargs.get("json"), default=json_serializer))

        try:
            return self.session.put(url, **kwargs)
        except requests.exceptions.ConnectionError as e:
            raise ConnectionError(self._connection_error_message(url, e))

    def post(self, url, **kwargs):
        if kwargs.get("json"):
            kwargs["json"] = json.loads(json.dumps(kwargs.get("json"), default=json_serializer))

        try:
            return self.session.post(url, **kwargs)
        except requests.exceptions.ConnectionError as e:
            raise ConnectionError(self._connection_error_message(url, e))

    def create(self, url, **kwargs):
        return self.post(url, **kwargs)

    def delete(self, url, **kwargs):
        if kwargs.get("json"):
            kwargs["json"] = json.loads(json.dumps(kwargs.get("json"), default=json_serializer))

        try:
            return self.session.delete(url, **kwargs)
        except requests.exceptions.ConnectionError as e:
            raise ConnectionError(self._connection_error_message(url, e))


client = Client()



from pathlib import Path
from typing import List, Optional, Union

from askanna.core.dataclasses.package import Package
from askanna.core.download import ChunkedDownload
from askanna.core.exceptions import GetError
from askanna.gateways.api_client import client

from .utils import ListResponse


class PackageListResponse(ListResponse):
    def __init__(self, data: dict):
        super().__init__(data)
        self.results: List[Package] = [Package.from_dict(package) for package in data["results"]]

    @property
    def packages(self):
        return self.results


class PackageGateway:
    """Management of packages in AskAnna
    This is the class which act as gateway to the API of AskAnna
    """

    def list(
        self,
        project_suuid: Optional[str] = None,
        workspace_suuid: Optional[str] = None,
        created_by_name: Optional[str] = None,
        created_by_suuid: Optional[str] = None,
        page_size: Optional[int] = None,
        cursor: Optional[str] = None,
        order_by: Optional[str] = None,
        search: Optional[str] = None,
    ) -> PackageListResponse:
        """Get a list of packages

        Args:
            project_suuid (str, optional): Project SUUID to filter for packages in a project. Defaults to None.
            workspace_suuid (str, optional): Workspace SUUID to filter for packages in a workspace. Defaults to None.
            created_by_name (str, optional): Filter packages on a created by name. Defaults to None.
            created_by_suuid (str, optional): Filter packages on a created by SUUID. Defaults to None.
            page_size (int, optional): Number of packages to return per page. Defaults to the default value of
                the backend.
            cursor (str, optional): Cursor to start the page from. Defaults to None.
            order_by (str, optional): Order by field(s). Defaults to the default value of the backend.
            search (str, optional): Search for a specific package.

        Raises:
            GetError: Error based on response status code with the error message from the API

        Returns:
            PackageListResponse: The response from the API with a list of packages and pagination information
        """
        assert page_size is None or page_size > 0, "page_size must be a positive integer"

        response = client.get(
            url=client.askanna_url.package.package_list(),
            params={
                "project_suuid": project_suuid,
                "workspace_suuid": workspace_suuid,
                "created_by_name": created_by_name,
                "created_by_suuid": created_by_suuid,
                "page_size": page_size,
                "cursor": cursor,
                "order_by": order_by,
                "search": search,
            },
        )

        if response.status_code != 200:
            error_message = f"{response.status_code} - Something went wrong while retrieving package list"
            try:
                error_message += f":\n  {response.json()}"
            except ValueError:
                pass
            raise GetError(error_message)

        return PackageListResponse(response.json())

    def detail(self, package_suuid: str) -> Package:
        """Get information of a package

        Args:
            package_suuid (str): SUUID of the package

        Raises:
            GetError: Error based on response status code with the error message from the API

        Returns:
            Package: Package information in a Package dataclass
        """

        response = client.get(
            url=client.askanna_url.package.package_detail(package_suuid),
        )

        if response.status_code == 404:
            raise GetError(f"404 - The package SUUID '{package_suuid}' was not found")
        if response.status_code != 200:
            raise GetError(
                f"{response.status_code} - Something went wrong while retrieving package SUUID '{package_suuid}': "
                f"{response.json()}"
            )

        return Package.from_dict(response.json())

    def download(self, package_suuid: str, output_path: Optional[Union[Path, str]] = None) -> Union[bytes, None]:
        """Download a package

        Args:
            package_suuid (str): SUUID of the package
            output_path (Path | str, optional): Path to save the package to. Defaults to None.

        Raises:
            GetError: Error based on response status code with the error message from the API

        Returns:
            bytes or None: The package as bytes or None if output_path is set
        """
        url = client.askanna_url.package.package_download(package_suuid)
        response = client.get(url)

        if response.status_code == 404:
            raise GetError(f"404 - Package SUUID '{package_suuid}' was not found")
        elif response.status_code != 200:
            raise GetError(
                f"{response.status_code} - Something went wrong while retrieving the package SUUID '{package_suuid}': "
                + str(response.json())
            )

        download_url = response.json().get("target")

        if output_path:
            download = ChunkedDownload(download_url)

            if download.status_code == 404:
                raise GetError(f"404 - Package SUUID '{package_suuid}' was not found")
            if download.status_code != 200:
                raise GetError(
                    f"{download.status_code} - Something went wrong while retrieving the pacakage SUUID "
                    f"'{package_suuid}'"
                )

            download.download(output_path)
            return None
        else:
            response = client.get(download_url, stream=True)

            if response.status_code == 404:
                raise GetError(f"404 - Package SUUID '{package_suuid}' was not found")
            if response.status_code != 200:
                raise GetError(
                    f"{response.status_code} - Something went wrong while retrieving the package SUUID "
                    f"'{package_suuid}': {response.json()}"
                )

            return response.content



from typing import Optional
from urllib.parse import parse_qs, urlsplit


class ListResponse:
    def __init__(self, data: dict):
        self.total_count: int = data["count"]
        self.next_url: Optional[str] = data["next"]
        self.previous_url: Optional[str] = data["previous"]
        self.results: dict = data["results"]

    def get_cursor(self, url: str) -> str:
        query_params = parse_qs(urlsplit(url).query)
        return query_params["cursor"][0]

    @property
    def next_url_cursor(self) -> Optional[str]:
        return self.get_cursor(self.next_url) if self.next_url else None

    @property
    def previous_url_cursor(self) -> Optional[str]:
        return self.get_cursor(self.previous_url) if self.previous_url else None



import click

from askanna.sdk.track import metric_collector

HELP = """
Retrieve metrics generated with track_metric(s) and push these metrics to AskAnna
"""

SHORT_HELP = "Push metrics to AskAnna"


@click.command(help=HELP, short_help=SHORT_HELP)
@click.option("--force", "-f", is_flag=True, help="Force push metrics")
def cli(force):
    if len(metric_collector) > 0 or force:
        metric_collector.save(force=True)



import sys

import click

from askanna.cli.utils import ask_which_project, ask_which_variable, ask_which_workspace
from askanna.config import config
from askanna.config.utils import string_format_datetime
from askanna.core.dataclasses.project import Project
from askanna.sdk.project import ProjectSDK
from askanna.sdk.variable import VariableSDK


@click.group()
def cli1():
    pass


@cli1.command(help="List variables in AskAnna", short_help="List variables")
@click.option(
    "--project", "-p", "project_suuid", required=False, type=str, help="Project SUUID to list variables for a project"
)
@click.option(
    "--workspace",
    "-w",
    "workspace_suuid",
    required=False,
    type=str,
    help="Workspace SUUID to list variables related to a workspace",
)
@click.option("--search", "-s", required=False, type=str, help="Search for a specific variable")
def list(project_suuid, workspace_suuid, search):
    variable_sdk = VariableSDK()
    try:
        variables = variable_sdk.list(
            project_suuid=project_suuid,
            workspace_suuid=workspace_suuid,
            search=search,
            order_by="project.name,name",
        )
    except Exception as e:
        click.echo(f"Something went wrong while listing the variables:\n  {e}", err=True)
        sys.exit(1)

    if not variables:
        click.echo("We cannot find any variables.")
        sys.exit(0)

    if project_suuid:
        click.echo(f"The variables for project '{variables[0].project.name}' are:")
        click.echo("")
        click.echo("-------------------    -------------------------")
        click.echo("VARIABLE SUUID         VARIABLE NAME")
        click.echo("-------------------    -------------------------")
    if not project_suuid and workspace_suuid:
        click.echo(f"The variables for workspace '{variables[0].workspace.name}' are:")
    else:
        click.echo("")
        click.echo("-------------------    --------------------    -------------------    -------------------------")
        click.echo("PROJECT SUUID          PROJECT NAME            VARIABLE SUUID         VARIABLE NAME")
        click.echo("-------------------    --------------------    -------------------    -------------------------")

    for variable in variables:
        variable_name = f"{variable.name[:22]}..." if len(variable.name) > 25 else variable.name[:25]
        if project_suuid:
            click.echo(f"{variable.suuid}    {variable_name}")
        else:
            project_name = (
                f"{variable.project.name[:17]}..." if len(variable.project.name) > 20 else variable.project.name[:20]
            )
            click.echo(
                "{project_suuid}    {project_name}    {variable_suuid}    {variable_name}".format(
                    project_suuid=variable.project.suuid,
                    project_name=f"{project_name:20}",
                    variable_suuid=variable.suuid,
                    variable_name=variable_name,
                )
            )

    if len(variables) != variable_sdk.list_total_count:
        click.echo("")
        click.echo(f"Note: the first {len(variables):,} of {variable_sdk.list_total_count:,} variables are shown.")

    click.echo("")


@cli1.command(help="Get information about a variable", short_help="Get variable info")
@click.option("--id", "-i", "variable_suuid", required=False, type=str, help="Variable SUUID")
def info(variable_suuid):
    if variable_suuid:
        try:
            variable = VariableSDK().get(variable_suuid=variable_suuid)
        except Exception as e:
            click.echo(f"Something went wrong while getting the variable info:\n  {e}", err=True)
            sys.exit(1)
    else:
        project_suuid = config.project.project_suuid
        if not project_suuid:
            workspace = ask_which_workspace(question="From which workspace do you want to get a variable?")
            project = ask_which_project(
                question="From which project do you want to get a variable?", workspace_suuid=workspace.suuid
            )
            project_suuid = project.suuid

        variable = ask_which_variable(question="Which variable do you want to get?", project_suuid=project_suuid)

    print_list = [
        None,
        ("SUUID", variable.suuid),
        ("Name", variable.name),
        ("Value", variable.value),
        ("Masked", variable.is_masked),
        None,
        ("Project", variable.project.name),
        ("Project SUUID", variable.project.suuid),
        ("Workspace", variable.workspace.name),
        ("Workspace SUUID", variable.workspace.suuid),
        None,
        ("Created", variable.created_at.strftime(string_format_datetime)),
        ("Modified", variable.modified_at.strftime(string_format_datetime)),
        None,
    ]
    for item in print_list:
        if item is None:
            click.echo("")
        else:
            click.echo(f"{item[0] + ':':16} {item[1]}")


@cli1.command(help="Add a variable to a project in AskAnna", short_help="Add variable")
@click.option("--name", "-n", required=False, type=str, help="Name of the variable")
@click.option("--value", "-v", required=False, type=str, help="Value of the variable")
@click.option(
    "--masked/--not-masked",
    "-m",
    "masked",
    default=False,
    show_default=False,
    help="Set value to masked (default is not-masked)",
)
@click.option(
    "--project",
    "-p",
    "project_suuid",
    type=str,
    required=False,
    help="Project SUUID where this variable will be created",
)
def add(name, value, masked, project_suuid):
    """
    Add a variable to a project
    """

    project = None
    if project_suuid:
        project = ProjectSDK().get(project_suuid=project_suuid)
        click.echo(f"Selected project: {project.name}")
    elif config.project.project_suuid:
        project = ProjectSDK().get(project_suuid=config.project.project_suuid)
        if click.confirm(f'\nDo you want to create a variable for project "{project.name}"?'):
            project_suuid = project.suuid
            click.echo(f"Selected project: {project.name}")

    if not project_suuid:
        workspace = ask_which_workspace(question="In which workspace do you want to add a variable?")
        project = ask_which_project(
            question="In which project do you want to add a variable?", workspace_suuid=workspace.suuid
        )

    if not project:
        click.echo(
            "Something went wrong while retrieving the project information. Retrieved project is None.", err=True
        )
        sys.exit(1)

    if type(project) is not Project:
        click.echo(
            "Something went wrong while retrieving the project information. Retrieved project is not of type Project.",
            err=True,
        )

    confirm = False
    if not name:
        name = click.prompt("\nName of the new variable", type=str)
        confirm = True

    if not value:
        value = click.prompt("Value of the new variable", type=str)
        confirm = True

        if not masked:
            masked = click.prompt("Should the value be masked in logs and the web interface? [y/N]", type=bool)

    if confirm:
        click.confirm(f'\nDo you want to create the variable "{name}" in project "{project.name}"?', abort=True)

    try:
        variable = VariableSDK().add(
            project_suuid=project.suuid,
            name=name,
            value=value,
            is_masked=masked,
        )
    except Exception as e:
        click.echo(f"Something went wrong in creating the variable:\n  {e}", err=True)
        sys.exit(1)
    else:
        click.echo(
            f"\nYou succesfully created variable '{variable.name}' with SUUID '{variable.suuid}' in project "
            f"'{project.name}'"
        )
        sys.exit(0)


@cli1.command(help="Change a variable in AskAnna", short_help="Change variable")
@click.option("--id", "-i", "variable_suuid", required=False, type=str, help="Variable SUUID")
@click.option("--name", "-n", required=False, type=str, help="New name to set")
@click.option("--value", "-v", required=False, type=str, help="New value to set")
@click.option("--masked", "-m", required=False, type=bool, help="Set value to masked")
def change(variable_suuid, name, value, masked):
    """
    Change a variable name, value and if the value should set to be masked.
    We will only proceed when any of the name or value is set.
    """
    if variable_suuid:
        try:
            variable = VariableSDK().get(variable_suuid=variable_suuid)
        except Exception as e:
            click.echo(f"Something went wrong while getting the variable info:\n  {e}", err=True)
            sys.exit(1)
    else:
        project_suuid = config.project.project_suuid
        if not project_suuid:
            workspace = ask_which_workspace(question="From which workspace do you want to change a variable?")
            project = ask_which_project(
                question="From which project do you want to change a variable?", workspace_suuid=workspace.suuid
            )
            project_suuid = project.suuid

        variable = ask_which_variable(question="Which variable do you want to change?", project_suuid=project_suuid)

    if not any([name, value, masked]):
        if click.confirm("\nDo you want to change the name of the variable?"):
            name = click.prompt("New name of the variable", type=str)
        if click.confirm("\nDo you want to change the value of the run?"):
            value = click.prompt("New value of the variable", type=str)

        if not any([name, value]):
            click.echo("We did not change anything because you did not request to change a name or value.", err=True)
            sys.exit(0)

        click.confirm("\nDo you want to change the variable?", abort=True)

    if masked and variable.is_masked:
        click.echo(
            "This variable is currently masked. It is not possible to unmask a masked variable."
            "We skip the option to change the mask status of the variable.",
            err=True,
        )
        sys.exit(0)

    try:
        variable = VariableSDK().change(variable_suuid=variable.suuid, name=name, value=value, is_masked=masked)
    except Exception as e:
        if str(e).startswith("404"):
            click.echo(f"The variable SUUID '{variable_suuid}' was not found.", err=True)
        else:
            click.echo(f"Something went wrong while changing the variable SUUID '{variable_suuid}':\n  {e}", err=True)
        sys.exit(1)

    click.echo(f"\nYou succesfully changed variable '{variable.name}' with SUUID '{variable.suuid}'")


@cli1.command(help="Remove a variable in AskAnna", short_help="Remove variable")
@click.option("--id", "-i", "variable_suuid", type=str, required=False, help="Variable SUUID", prompt="Variable SUUID")
@click.option("--force", "-f", is_flag=True, help="Force")
def remove(variable_suuid, force):
    if variable_suuid:
        try:
            variable = VariableSDK().get(variable_suuid=variable_suuid)
        except Exception as e:
            if str(e).startswith("404"):
                click.echo(f"The variable SUUID '{variable_suuid}' was not found.", err=True)
            else:
                click.echo(
                    f"Something went wrong while getting the info of variable SUUID '{variable_suuid}':\n  {e}",
                    err=True,
                )
            sys.exit(1)
    else:
        project_suuid = config.project.project_suuid
        if not project_suuid:
            workspace = ask_which_workspace(question="From which workspace do you want to remove a variable?")
            project = ask_which_project(
                question="From which project do you want to remove a variable?", workspace_suuid=workspace.suuid
            )
            project_suuid = project.suuid

        variable = ask_which_variable(question="Which variable do you want to remove?", project_suuid=project_suuid)

    if not force:
        if not click.confirm(f'Are you sure to delete variable {variable.suuid} with name "{variable.name}"?'):
            click.echo("Understood. We are not removing the variable.")
            sys.exit(0)

    try:
        removed = VariableSDK().delete(variable_suuid=variable.suuid)
    except Exception as e:
        click.echo(f"Something went wrong while removing the variable SUUID '{variable_suuid}':\n  {e}", err=True)
        sys.exit(1)

    if removed:
        click.echo(f"You removed variable SUUID '{variable.suuid}'")
    else:
        click.echo(f"Something went wrong. Removing the run SUUID '{variable.suuid}' failed.", err=True)
        sys.exit(1)


cli = click.CommandCollection(
    sources=[cli1],
    help="Manage your variables in AskAnna",
    short_help="Manage variables in AskAnna",
)



import sys

import click

from askanna.config import config
from askanna.gateways.auth import AuthGateway

HELP = """
Add your AskAnna API key to your global configuration file (~/.askanna.yml). This is necessary to gain access to
projects associated with your AskAnna account.
"""

SHORT_HELP = "Login and save your AskAnna API token"


@click.command(help=HELP, short_help=SHORT_HELP)
@click.option("--email", "-e", required=False, type=str, help="Email which you use to login")
@click.option("--password", "-p", required=False, help="Password you use to login")
@click.option("--url", required=False, type=str, help="URL you want to login to")
@click.option("--remote", "-r", required=False, type=str, help="Remote you want to login to")
def cli(email: str, password: str, url: str, remote: str):
    auth = AuthGateway()

    if config.server.is_authenticated:
        # Showcase with this token, who we are and provide info what do to if you want to use another account
        user = auth.get_user_info()
        click.echo(f"You are already logged in with email '{user.email}'.")

        logout = click.confirm(f"Do you want to log out email '{user.email}' and log in with another account?")
        if logout:
            config.server.logout_and_remove_token()
        else:
            click.echo("Login with a new account aborted.")
            sys.exit(0)

    if email and password:
        pass
    else:
        click.echo("Let's login into AskAnna")
        if not email:
            email = click.prompt("Email", type=str)
        if not password:
            password = click.prompt("Password", type=str, hide_input=True)

    # Do the actual login and update the config file with the token
    try:
        auth.login(email=email, password=password, remote_url=remote, ui_url=url, update_config_file=True)
    except Exception as e:
        click.echo(f"Login to AskAnna failed:\n  {e}")
        sys.exit(1)

    user = auth.get_user_info()
    click.echo(f"You are logged in with email '{user.email}'")



import io
import sys
from pathlib import Path
from zipfile import ZipFile

import click

from askanna.sdk.package import PackageSDK

HELP = """
Get the code package for the run from AskAnna. Intended for use with runner and unpacks the package to a directory.
"""
SHORT_HELP = "Get code package from AskAnna"


@click.command(help=HELP, short_help=SHORT_HELP)
@click.option(
    "--package",
    "package_suuid",
    required=True,
    envvar="AA_PACKAGE_SUUID",
    type=str,
    help="The package SUUID",
)
@click.option(
    "--output",
    "output_dir",
    envvar="AA_CODE_DIR",
    default="/code",
    show_default=True,
    type=click.Path(path_type=Path),
)
def cli(package_suuid, output_dir):
    try:
        package_content = PackageSDK().get(package_suuid)
    except Exception as e:
        click.echo(f"Something went wrong getting the package. The error message received:\n  {e}", err=True)
        sys.exit(1)

    if not package_content:
        click.echo(f"No files found for package SUUID '{package_suuid}'", err=True)
        sys.exit(1)

    output_dir.mkdir(parents=True, exist_ok=True)
    with ZipFile(io.BytesIO(package_content), "r") as zip_file:
        zip_file.extractall(output_dir)



import os
import sys
from typing import Dict

import click
import requests
import yaml

try:
    from yaml import CSafeDumper as SafeDumper
    from yaml import CSafeLoader as SafeLoader
except ImportError:  # pragma: no cover
    from yaml import SafeDumper, SafeLoader


string_format_datetime = "%Y-%m-%d %H:%M:%S %Z"


def read_config(path: str) -> Dict:
    """
    Reading existing config or return default dict
    """
    try:
        with open(os.path.expanduser(path)) as f:
            return yaml.load(f, Loader=SafeLoader) or {}
    except FileNotFoundError:
        click.echo(f"Cannot find the config file: {os.path.expanduser(path)}", err=True)
        sys.exit(1)
    except TypeError as e:
        click.echo(e, err=True)
        sys.exit(1)
    except yaml.scanner.ScannerError as e:
        click.echo("Error reading 'askanna.yml' due to:", err=True)
        click.echo(e.problem, err=True)
        click.echo(e.problem_mark, err=True)
        sys.exit(1)


def store_config(path, config: Dict):
    with open(path, "w") as f:
        f.write(yaml.dump(config, Dumper=SafeDumper))


def scan_config_in_path(path: str = ""):
    """
    Look for file `askanna.yml` in parent directories
    """
    filename = "askanna.yml"
    project_configfile = ""

    if not path:
        path = os.getcwd()

    # first check whether we already can find in the current workdir
    if contains_configfile(path, filename):
        project_configfile = os.path.join(path, filename)
    else:
        # traverse up all directories untill we find an askanna.yml file
        split_path = os.path.split(path)
        # in any other cases, look in parent directories
        while split_path[1] != "":
            if contains_configfile(split_path[0], filename):
                project_configfile = os.path.join(split_path[0], filename)
                break
            split_path = os.path.split(split_path[0])
    return project_configfile


def contains_configfile(path: str, filename: str = "askanna.yml") -> bool:
    return os.path.isfile(os.path.join(path, filename))


def read_config_from_url(url: str) -> Dict:
    """
    Get config from URL and return it as dict
    """
    try:
        r = requests.get(url)
    except requests.exceptions.ConnectionError:
        click.echo(f"Cannot open URL: {url}", err=True)
        sys.exit(1)

    if not r.headers.get("content-type") == "application/octet-stream":
        click.echo(f"The URL '{url}' does not return an expected YAML config", err=True)
        sys.exit(1)

    try:
        return yaml.load(r.content, Loader=SafeLoader) or {}
    except yaml.scanner.ScannerError as e:
        click.echo("Error reading YAML content due to:", err=True)
        click.echo(e.problem, err=True)
        click.echo(e.problem_mark, err=True)
        sys.exit(1)
